/*************************************************************************
    amplify.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <vgagl.h>
#include "multitrack.h"

#include "images/ok_up.xpm"
#include "images/ok_down.xpm"
#include "images/cancel_up.xpm"
#include "images/cancel_down.xpm"
#include "images/check_up.xpm"
#include "images/check_down.xpm"


void amplify_track(float initial, float final)
{
	int output_fd, input_fd, max_slicelen = 16384, slicelen, i;
	long begin = screen_to_sample(select_begin);
	long end = screen_to_sample(select_end), position, length;
	short int *data;
	float step, volume;
	char filename[80];

	sprintf(filename, "track%d.wav", edit_track);
	input_fd = open(filename, O_RDONLY);
	if (input_fd == -1)
		error("Cannot open track for reading.");
	lseek(input_fd, begin * 2, SEEK_SET);

	output_fd = open(filename, O_WRONLY);
	if (output_fd == -1)
		error("Cannot open track for writing.");
	lseek(output_fd, begin * 2, SEEK_SET);

	data = (short int *) malloc(sizeof(short int) * max_slicelen);
	if (data == NULL)
		error("Unable to allocate memory in function: amplify.");

	position = 0;
	length = end - begin + 1;
	step = (final - initial) / (float) length;
	volume = initial;
	init_progress("Amplifying selection.");
	do {
		slicelen = length - position;
		if (slicelen > max_slicelen)
			slicelen = max_slicelen;

		read(input_fd, (char *) data, slicelen * 2);
		for (i = 0; i < slicelen; i++) {
			data[i] = (short int) (data[i] * volume);
			volume += step;
		}
		write(output_fd, (char *) data, slicelen * 2);
		position += slicelen;
		show_progress(position / (float) length);
	}
	while (slicelen == max_slicelen);
	delete_progress();

	close(output_fd);
	close(input_fd);

	make_display_file(edit_track, begin, end);
}


static Button winbutton[6];
static Slider winslider[2];
static Window win;

static void winfunction(void)
{
}


static void show_value(int slider_id)
{
	int x, y;
	char hulp[10];

	sprintf(hulp, "%0.2f", winslider[slider_id].position / 100.0);
	x = winslider[slider_id].x - 8;
	y = winslider[slider_id].y - 13;
	gl_fillbox(x, y, 32, 7, DARKGREY);
	gl_write(x, y, hulp);
}


static void winslider_cb(int slider_id)
{
	if (winbutton[3].pressed) {	/* lock pressed */
		if (slider_id == 0) {
			winslider[1].position = winslider[0].position;
			show_slider(winslider[1], FALSE);
			show_value(1);
		} else {
			winslider[0].position = winslider[1].position;
			show_slider(winslider[0], FALSE);
			show_value(0);
		}
	}
	show_value(slider_id);
}


static void check_cb(int button_id)
{
	int count;

	for (count = 0; count < 4; count++)
		show_button(&winbutton[count], FALSE);

	switch (button_id) {
	case 0:
		show_button(&winbutton[0], TRUE);
		winslider[0].position = 0;
		winslider[1].position = 100;
		break;
	case 1:
		show_button(&winbutton[1], TRUE);
		winslider[0].position = 100;
		winslider[1].position = 0;
		break;
	case 2:
		show_button(&winbutton[2], TRUE);
		show_button(&winbutton[3], TRUE);
		winslider[0].position = 80;
		winslider[1].position = 80;
		break;
	}
	if (button_id < 3) {
		show_slider(winslider[0], FALSE);
		show_value(0);
		show_slider(winslider[1], FALSE);
		show_value(1);
	}
	wait_for_mouse();
}


static void lock_cb(int data)
{
	winslider[1].position = winslider[0].position;
	show_slider(winslider[1], FALSE);
	show_value(1);
	wait_for_mouse();
}


void amplify_menu(void)
{
	Window_text text[6];

	win.x = 280;
	win.y = 225;
	win.width = 180;
	win.height = 130;
	win.function = winfunction;

	winbutton[0].x = win.x + 5;
	winbutton[0].y = win.y + 5;
	translate_XPM(&winbutton[0], check_up_xpm, FALSE);
	translate_XPM(&winbutton[0], check_down_xpm, TRUE);
	winbutton[0].pressed = FALSE;
	winbutton[0].active = TRUE;
	winbutton[0].function = check_cb;

	winbutton[1].x = win.x + 5;
	winbutton[1].y = win.y + 25;
	translate_XPM(&winbutton[1], check_up_xpm, FALSE);
	translate_XPM(&winbutton[1], check_down_xpm, TRUE);
	winbutton[1].pressed = FALSE;
	winbutton[1].active = TRUE;
	winbutton[1].function = check_cb;

	winbutton[2].x = win.x + 5;
	winbutton[2].y = win.y + 45;
	translate_XPM(&winbutton[2], check_up_xpm, FALSE);
	translate_XPM(&winbutton[2], check_down_xpm, TRUE);
	winbutton[2].pressed = TRUE;
	winbutton[2].active = TRUE;
	winbutton[2].function = check_cb;

	winbutton[3].x = win.x + 5;
	winbutton[3].y = win.y + 85;
	translate_XPM(&winbutton[3], check_up_xpm, FALSE);
	translate_XPM(&winbutton[3], check_down_xpm, TRUE);
	winbutton[3].pressed = TRUE;
	winbutton[3].active = TRUE;
	winbutton[3].function = lock_cb;

	winbutton[4].x = win.x + 5;
	winbutton[4].y = win.y + win.height - 20;
	translate_XPM(&winbutton[4], cancel_up_xpm, FALSE);
	translate_XPM(&winbutton[4], cancel_down_xpm, TRUE);
	winbutton[4].pressed = FALSE;
	winbutton[4].active = TRUE;
	winbutton[4].function = quit_cb;

	winbutton[5].x = win.x + 55;
	winbutton[5].y = win.y + win.height - 20;
	translate_XPM(&winbutton[5], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[5], ok_down_xpm, TRUE);
	winbutton[5].pressed = FALSE;
	winbutton[5].active = TRUE;
	winbutton[5].function = quit_cb;

	winslider[0].x = win.x + 110;
	winslider[0].y = win.y + 25;
	winslider[0].active = TRUE;
	winslider[0].length = 100;
	winslider[0].position = 80;
	winslider[0].function = winslider_cb;

	winslider[1].x = win.x + 150;
	winslider[1].y = win.y + 25;
	winslider[1].active = TRUE;
	winslider[1].length = 100;
	winslider[1].position = 80;
	winslider[1].function = winslider_cb;

	text[0].x = win.x + 25;
	text[0].y = win.y + 8;
	sprintf(text[0].string, "Fade in");
	text[1].x = win.x + 25;
	text[1].y = win.y + 28;
	sprintf(text[1].string, "Fade out");
	text[2].x = win.x + 25;
	text[2].y = win.y + 48;
	sprintf(text[2].string, "Amplify");
	text[3].x = win.x + 25;
	text[3].y = win.y + 88;
	sprintf(text[3].string, "Lock I-F");
	text[4].x = win.x + 114;
	text[4].y = win.y + 2;
	sprintf(text[4].string, "I");
	text[5].x = win.x + 154;
	text[5].y = win.y + 2;
	sprintf(text[5].string, "F");

	if (do_window(&win, winbutton, 6, winslider, 2, text, 6, 5, 4)) {
		amplify_track(winslider[0].position / 100.0, winslider[1].position / 100.0);
		delete_mouse();
		show_tracks();
		select_region(select_begin, select_end, TRUE);
		show_mouse();
	}
	show_button(&button[AMPLIFY], FALSE);
}
