/*************************************************************************
    buffer.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"

#include "images/ok_up.xpm"
#include "images/ok_down.xpm"
#include "images/cancel_up.xpm"
#include "images/cancel_down.xpm"
#include "images/check_up.xpm"
#include "images/check_down.xpm"

static Window win;
static Button winbutton[4];


static void copy_track_to_buffer(void)
{
	long begin = screen_to_sample(select_begin);
	long end = screen_to_sample(select_end);
	long max_length = 22050 * 24;
	int fd;
	char filename[80];

	bigbuf_info->filled = TRUE;
	bigbuf_info->length = end - begin + 1;
	if (bigbuf_info->length > max_length)
		bigbuf_info->length = max_length;
	bigbuf_info->position = begin;

	sprintf(filename, "track%d.wav", edit_track);
	fd = open(filename, O_RDONLY);
	if (fd == -1)
		error("Cannot open track.");
	lseek(fd, begin * 2, SEEK_SET);

	read(fd, (char *) bigbuf, bigbuf_info->length * 2);
	close(fd);
}


static void winfunction(void)
{
}


void overload_menu(char *sort)
{
	Button winbutton[1];
	Window win;
	Window_text text[2];

	win.x = 270;
	win.y = 225;
	win.height = 60;
	win.width = 170;
	win.function = winfunction;

	winbutton[0].x = win.x + 60;
	winbutton[0].y = win.y + 35;
	translate_XPM(&winbutton[0], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[0], ok_down_xpm, TRUE);
	winbutton[0].pressed = FALSE;
	winbutton[0].active = TRUE;
	winbutton[0].function = quit_cb;

	text[0].x = win.x + 14;
	text[0].y = win.y + 5;
	sprintf(text[0].string, "Overload detected!");
	text[1].x = win.x + 10;
	text[1].y = win.y + 15;
	sprintf(text[1].string, "Abort overlap %s", sort);

	do_window(&win, winbutton, 1, NULL, 0, text, 2, 0, -2);
}


static int check_overload(float rel_volume)
/* Check if result for overlap paste does not exceed maximum or minimum
   value */
{
	int input_fd, overload = FALSE, i;
	long position = bigbuf_info->position, count;
	char filename[80], buffer[2];

	sprintf(filename, "display%d.dat", edit_track);
	input_fd = open(filename, O_RDONLY);
	if (input_fd == -1)
		error("Cannot open display-file.");

	lseek(input_fd, (position / 512) * 2, SEEK_SET);
	count = 0;
	do {
		read(input_fd, buffer, 2);
		for (i = 0; i < 512; i++) {
			if (((bigbuf[count] >> 8) * rel_volume + buffer[0] < -127) ||
			    ((bigbuf[count] >> 8) * rel_volume + buffer[1] > 128))
				overload = TRUE;
			count++;
		}
	}
	while (!overload && count < bigbuf_info->length);

	return overload;
}


static void copy_buffer_to_track(int overlap)
{
	int output_fd, input_fd, slicelen, changed;
	int max_slicelen = 16384, i, track_id = 0;
	short int *data;
	char filename[80];
	long position;
	float rel_volume;

	sprintf(filename, "track%d.wav", edit_track);
	output_fd = open(filename, O_WRONLY);
	if (output_fd == -1)
		error("Cannot open track for writing.");
	input_fd = open(filename, O_RDONLY);
	if (input_fd == -1)
		error("Cannot open track for reading.");
	data = (short int *) malloc(sizeof(short int) * max_slicelen);
	if (data == NULL)
		error("Unable to allocate memory.");

	lseek(output_fd, bigbuf_info->position * 2, SEEK_SET);
	lseek(input_fd, bigbuf_info->position * 2, SEEK_SET);

	changed = FALSE;
	if (!overlap) {
		write(output_fd, (char *) bigbuf, bigbuf_info->length * 2);
		changed = TRUE;
	} else {
		track_id = bigbuf_info->show_buffer;
		rel_volume = slider[VOL_SLIDE + track_id].position /
		    (float) slider[VOL_SLIDE + edit_track].position;
		if (!check_overload(rel_volume)) {
			init_progress("Copy buffer to track.");

			position = 0;
			do {
				slicelen = bigbuf_info->length - position;
				if (slicelen > max_slicelen)
					slicelen = max_slicelen;

				read(input_fd, (char *) data, slicelen * 2);
				for (i = 0; i < slicelen; i++)
					data[i] += (short int) (bigbuf[i + position] * rel_volume);
				write(output_fd, (char *) data, slicelen * 2);
				position += slicelen;
				show_progress(position / (float) bigbuf_info->length);
			}
			while (slicelen == max_slicelen);

			delete_progress();
			changed = TRUE;
		} else
			overload_menu("paste");
	}

	close(output_fd);
	close(input_fd);
	if (changed)
		make_display_file(edit_track, bigbuf_info->position, bigbuf_info->position + bigbuf_info->length);
}


static void cut_track(int to_buf)
{
	long begin = screen_to_sample(select_begin);
	long end = screen_to_sample(select_end), length;
	int fd, max_slicelen = 16384, slicelen;
	long written;
	char filename[80];
	short int *data;

	if (to_buf) {
		copy_track_to_buffer();
		length = bigbuf_info->length;
	} else
		length = end - begin + 1;

	sprintf(filename, "track%d.wav", edit_track);
	fd = open(filename, O_WRONLY);
	if (fd == -1)
		error("Cannot open track for writing.");

	lseek(fd, begin * 2, SEEK_SET);
	data = (short int *) malloc(sizeof(short int) * max_slicelen);
	if (data == NULL)
		error("Unable to allocate memory in function: cut_track.");
	bzero(data, max_slicelen * 2);

	init_progress("Cutting track.");
	written = 0;
	do {
		slicelen = length - written;
		if (slicelen > max_slicelen)
			slicelen = max_slicelen;
		write(fd, (char *) data, slicelen * 2);
		written += slicelen;
		show_progress(written / (float) length);
	}
	while (slicelen == max_slicelen);
	delete_progress();

	close(fd);
	make_display_file(edit_track, begin, end);
}




static void check_cb(int button_id)
{
	if (button_id == 2) {
		show_button(&winbutton[2], TRUE);
		show_button(&winbutton[3], FALSE);
	}
	if (button_id == 3) {
		show_button(&winbutton[2], FALSE);
		show_button(&winbutton[3], TRUE);
	}
}


static void init_window(void)
{
	win.x = 270;
	win.y = 225;
	win.width = 150;
	win.height = 70;
	win.function = winfunction;

	winbutton[0].x = win.x + 100;
	winbutton[0].y = win.y + 50;
	translate_XPM(&winbutton[0], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[0], ok_down_xpm, TRUE);
	winbutton[0].pressed = FALSE;
	winbutton[0].active = TRUE;
	winbutton[0].function = quit_cb;

	winbutton[1].x = win.x + 10;
	winbutton[1].y = win.y + 50;
	translate_XPM(&winbutton[1], cancel_up_xpm, FALSE);
	translate_XPM(&winbutton[1], cancel_down_xpm, TRUE);
	winbutton[1].pressed = FALSE;
	winbutton[1].active = TRUE;
	winbutton[1].function = quit_cb;

	winbutton[2].x = win.x + 10;
	winbutton[2].y = win.y + 5;
	translate_XPM(&winbutton[2], check_up_xpm, FALSE);
	translate_XPM(&winbutton[2], check_down_xpm, TRUE);
	winbutton[2].pressed = TRUE;
	winbutton[2].active = TRUE;
	winbutton[2].function = check_cb;

	winbutton[3].x = win.x + 10;
	winbutton[3].y = win.y + 25;
	translate_XPM(&winbutton[3], check_up_xpm, FALSE);
	translate_XPM(&winbutton[3], check_down_xpm, TRUE);
	winbutton[3].pressed = FALSE;
	winbutton[3].active = TRUE;
	winbutton[3].function = check_cb;
}


void cut_menu(void)
{
	Window_text text[2];

	init_window();
	text[0].x = win.x + 30;
	text[0].y = win.y + 10;
	sprintf(text[0].string, "Cut to buffer");
	text[1].x = win.x + 30;
	text[1].y = win.y + 30;
	sprintf(text[1].string, "Delete");

	if (do_window(&win, winbutton, 4, NULL, 0, text, 2, 0, 1)) {
		if (winbutton[2].pressed)	/* copy to buffer checked */
			cut_track(TRUE);
		else
			cut_track(FALSE);	/* delete checked */
		delete_mouse();
		show_tracks();
		select_region(select_begin, select_end, TRUE);
		show_mouse();
	}
}


void paste_menu(void)
{
	Window_text text[2];

	init_window();
	text[0].x = win.x + 30;
	text[0].y = win.y + 10;
	sprintf(text[0].string, "Normal paste");
	text[1].x = win.x + 30;
	text[1].y = win.y + 30;
	sprintf(text[1].string, "Overlap paste");

	if (do_window(&win, winbutton, 4, NULL, 0, text, 2, 0, 1)) {
		if (winbutton[2].pressed)	/* normal paste checked */
			copy_buffer_to_track(FALSE);
		else
			copy_buffer_to_track(TRUE);
		delete_mouse();
		show_tracks();
		select_region(select_begin, select_end, TRUE);
		show_mouse();
	}
}


void copy_menu(void)
{
	Window_text text[2];

	init_window();
	text[0].x = win.x + 30;
	text[0].y = win.y + 10;
	sprintf(text[0].string, "Copy to buffer");
	text[1].x = win.x + 30;
	text[1].y = win.y + 30;
	sprintf(text[1].string, "Copy to track");

	if (do_window(&win, winbutton, 4, NULL, 0, text, 2, 0, 1)) {
		if (winbutton[2].pressed)	/* normal paste checked */
			copy_track_to_buffer();
		else
			copy_menu2();
		delete_mouse();
		show_tracks();
		select_region(select_begin, select_end, TRUE);
		show_mouse();
	}
}


void show_buffer(int track_id)
{
	int x, i, upper, lower, element, step, teller, bg_color;
	char buffer[2];
	long position;

	step = (int) ((track_end - track_begin + 1) / (float) screen_length);
	teller = 0;
	for (x = screen_min; x <= screen_max; x++) {
		position = screen_to_sample(x) - bigbuf_info->position;
		if (position >= 0 && position <= bigbuf_info->length) {
			max_min(bigbuf + position, step, buffer);
			teller++;
			bg_color = DARKGREY;
		} else {
			buffer[0] = 0;
			buffer[1] = 0;
			bg_color = BLACK;
		}
		/* draw line */
		lower = (int) ((buffer[0] / 128.0 + 1) * screen[track_id].height / 2);
		upper = (int) ((buffer[1] / 128.0 + 1) * screen[track_id].height / 2);
		for (i = 0; i <= screen[track_id].height; i++) {
			element = i * screen_length + x - screen_min;
			if (i >= lower && i <= upper)
				screen[track_id].data[element] = GREEN;
			else
				screen[track_id].data[element] = bg_color;
		}
	}
}
