/*************************************************************************
    button.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <vga.h>
#include <vgagl.h>
#include "multitrack.h"

#include "button.h"

static void check_buttons(void)
{
	int count;

	for (count = 0; count < nr_buttons; count++)
		button[count].active = TRUE;

	if (!bigbuf_info->filled) {
		button[PASTE].active = FALSE;
		for (count = 0; count < nr_tracks; count++)
			button[SWITCH + count].active = FALSE;
	}
	if (bigbuf_info->show_buffer == -1)
		button[PASTE].active = FALSE;
	else {
		button[PFL + bigbuf_info->show_buffer].active = FALSE;
		button[MUTE + bigbuf_info->show_buffer].active = FALSE;
		button[EDIT + bigbuf_info->show_buffer].active = FALSE;
		button[REC + bigbuf_info->show_buffer].active = FALSE;
	}
	if (select_begin == select_end || edit_track == -1) {
		button[CUT].active = FALSE;
		button[COPY].active = FALSE;
		button[AMPLIFY].active = FALSE;
	}
	if (info->playing || info->recording) {
		button[ZOOMIN].active = FALSE;
		button[ZOOMOUT].active = FALSE;
		button[ZOOM].active = FALSE;
		button[SEND].active = FALSE;
		button[PLAY].active = FALSE;
		button[RECORD].active = FALSE;
		button[CUT].active = FALSE;
		button[PASTE].active = FALSE;
		button[COPY].active = FALSE;
		button[AMPLIFY].active = FALSE;
		button[QUIT].active = FALSE;
		for (count = 0; count < nr_tracks; count++) {
			button[EDIT + count].active = FALSE;
			button[REC + count].active = FALSE;
			button[SWITCH + count].active = FALSE;
		}
	}
}


int button_region(Button * but, int number_buttons)
{
	int button_pressed = -1, i;

	for (i = 0; i < number_buttons; i++)
		if (mouse_x >= but[i].x && mouse_x <= but[i].x + but[i].width &&
		    mouse_y >= but[i].y && mouse_y <= but[i].y + but[i].height)
			button_pressed = i;

	return button_pressed;
}


int press_button(Button * but, int number_buttons)
{
	int button_pressed = button_region(but, number_buttons);

	check_buttons();
	if (button_pressed != -1 && but[button_pressed].active) {
		if (but[button_pressed].pressed)
			show_button(&but[button_pressed], FALSE);
		else
			show_button(&but[button_pressed], TRUE);
		but[button_pressed].function(button_pressed);
	} else
		button_pressed = -1;

	return button_pressed;
}


static void display_button(Button * but, int pressed)
{
	but->pressed = pressed;
	if (but->active)
		gl_putbox(but->x, but->y, but->width, but->height, but->data[pressed]);
}


void show_button(Button * but, int pressed)
{
	delete_mouse();
	display_button(but, pressed);
	show_mouse();
}


void translate_XPM(Button * but, char **data, int pressed)
{
	char buffer[80], character[10];
	int color[10], num_colors, i, count, row, column, width, height;
	char *color_string[9] =
	{"white", "black", "grey", "darkgrey", "green",
	 "red", "yellow", "blue", "lightblue"};
	int color_int[9] =
	{WHITE, BLACK, GREY, DARKGREY, GREEN, RED, YELLOW, BLUE, LIGHTBLUE};

	sscanf(data[0], "%d %d %d %*s\n", &width, &height, &num_colors);
	for (count = 0; count < num_colors; count++) {
		sscanf(data[count + 1], "%c%*cc %[a-z]s", &character[count], buffer);
		for (i = 0; i < 9; i++)
			if (strcmp(buffer, color_string[i]) == 0)
				color[count] = color_int[i];
	}
	but->data[pressed] = (char *) malloc(sizeof(char) * width * height);
	if (but->data[pressed] == NULL)
		error("Unable to allocate data array for pixmap.");

	for (row = 0; row < height; row++) {
		for (column = 0; column < width; column++) {
			for (i = 0; i < num_colors; i++)
				if (data[row + num_colors + 1][column] == character[i])
					but->data[pressed][row * width + column] = color[i];
		}
	}
	but->width = width;
	but->height = height;
	but->pressed = FALSE;
	but->active = TRUE;
}


void show_track_buttons(int track_id)
{
	display_button(&button[PFL + track_id], button[PFL + track_id].pressed);
	display_button(&button[MUTE + track_id], button[MUTE + track_id].pressed);
	display_button(&button[EDIT + track_id], button[EDIT + track_id].pressed);
	display_button(&button[REC + track_id], button[REC + track_id].pressed);
	display_button(&button[SWITCH + track_id], button[SWITCH + track_id].pressed);
}


void init_buttons(void)
{
	int button_id, count;

	button[ZOOMIN].x = 2;
	button[ZOOMIN].y = 2;
	translate_XPM(&button[ZOOMIN], zoomin_up_xpm, FALSE);
	translate_XPM(&button[ZOOMIN], zoomin_down_xpm, TRUE);
	button[ZOOMIN].function = zoomin_cb;
	sprintf(button[ZOOMIN].info, "Zoom in");

	button[ZOOMOUT].x = 26;
	button[ZOOMOUT].y = 2;
	translate_XPM(&button[ZOOMOUT], zoomout_up_xpm, FALSE);
	translate_XPM(&button[ZOOMOUT], zoomout_down_xpm, TRUE);
	button[ZOOMOUT].function = zoomout_cb;
	sprintf(button[ZOOMOUT].info, "Zoom out");

	button[ZOOM].x = 50;
	button[ZOOM].y = 2;
	translate_XPM(&button[ZOOM], zoom_up_xpm, FALSE);
	translate_XPM(&button[ZOOM], zoom_down_xpm, TRUE);
	button[ZOOM].function = zoom_cb;
	sprintf(button[ZOOM].info, "Zoom");

	button[LOAD].x = 90;
	button[LOAD].y = 2;
	translate_XPM(&button[LOAD], load_up_xpm, FALSE);
	translate_XPM(&button[LOAD], load_down_xpm, TRUE);
	button[LOAD].function = load_cb;
	sprintf(button[LOAD].info, "Load");

	button[SAVE].x = 114;
	button[SAVE].y = 2;
	translate_XPM(&button[SAVE], save_up_xpm, FALSE);
	translate_XPM(&button[SAVE], save_down_xpm, TRUE);
	button[SAVE].function = save_cb;
	sprintf(button[SAVE].info, "Save");

	button[SEND].x = 138;
	button[SEND].y = 2;
	translate_XPM(&button[SEND], send_up_xpm, FALSE);
	translate_XPM(&button[SEND], send_down_xpm, TRUE);
	button[SEND].function = send_cb;
	sprintf(button[SEND].info, "Send");

	button[REW].x = 196;
	button[REW].y = 2;
	translate_XPM(&button[REW], rew_up_xpm, FALSE);
	translate_XPM(&button[REW], rew_down_xpm, TRUE);
	button[REW].function = rew_cb;

	button[STOP].x = 220;
	button[STOP].y = 2;
	translate_XPM(&button[STOP], stop_up_xpm, FALSE);
	translate_XPM(&button[STOP], stop_down_xpm, TRUE);
	button[STOP].function = stop_cb;

	button[PLAY].x = 244;
	button[PLAY].y = 2;
	translate_XPM(&button[PLAY], play_up_xpm, FALSE);
	translate_XPM(&button[PLAY], play_down_xpm, TRUE);
	button[PLAY].function = play_cb;

	button[FF].x = 268;
	button[FF].y = 2;
	translate_XPM(&button[FF], ff_up_xpm, FALSE);
	translate_XPM(&button[FF], ff_down_xpm, TRUE);
	button[FF].function = ff_cb;

	button[RECORD].x = 292;
	button[RECORD].y = 2;
	translate_XPM(&button[RECORD], record_up_xpm, FALSE);
	translate_XPM(&button[RECORD], record_down_xpm, TRUE);
	button[RECORD].function = record_cb;
	sprintf(button[RECORD].info, "Record");

	button[MIXER0].x = 350;
	button[MIXER0].y = 2;
	translate_XPM(&button[MIXER0], mixer0_up_xpm, FALSE);
	translate_XPM(&button[MIXER0], mixer0_down_xpm, TRUE);
	button[MIXER0].function = mixer0_cb;
	sprintf(button[MIXER0].info, "Mixer settings");

	button[MIXER1].x = 384;
	button[MIXER1].y = 2;
	translate_XPM(&button[MIXER1], mixer1_up_xpm, FALSE);
	translate_XPM(&button[MIXER1], mixer1_down_xpm, TRUE);
	button[MIXER1].function = mixer1_cb;
	sprintf(button[MIXER1].info, "Mixer settings");

	button[CUT].x = 440;
	button[CUT].y = 2;
	translate_XPM(&button[CUT], cut_up_xpm, FALSE);
	translate_XPM(&button[CUT], cut_down_xpm, TRUE);
	button[CUT].function = cut_cb;
	sprintf(button[CUT].info, "Cut");

	button[PASTE].x = 464;
	button[PASTE].y = 2;
	translate_XPM(&button[PASTE], paste_up_xpm, FALSE);
	translate_XPM(&button[PASTE], paste_down_xpm, TRUE);
	button[PASTE].function = paste_cb;
	sprintf(button[PASTE].info, "Paste");

	button[COPY].x = 488;
	button[COPY].y = 2;
	translate_XPM(&button[COPY], copy_up_xpm, FALSE);
	translate_XPM(&button[COPY], copy_down_xpm, TRUE);
	button[COPY].function = copy_cb;
	sprintf(button[COPY].info, "Copy");

	button[AMPLIFY].x = 512;
	button[AMPLIFY].y = 2;
	translate_XPM(&button[AMPLIFY], amplify_up_xpm, FALSE);
	translate_XPM(&button[AMPLIFY], amplify_down_xpm, TRUE);
	button[AMPLIFY].function = amplify_cb;
	sprintf(button[AMPLIFY].info, "Amplify");

	button[QUIT].x = 593;
	button[QUIT].y = 2;
	translate_XPM(&button[QUIT], quit_up_xpm, FALSE);
	translate_XPM(&button[QUIT], quit_down_xpm, TRUE);
	button[QUIT].function = quit_cb;

	for (count = 0; count < nr_tracks; count++) {
		button_id = PFL + count;
		button[button_id].x = 5;
		button[button_id].y = screen[count].position + 2;
		translate_XPM(&button[button_id], pfl_up_xpm, FALSE);
		translate_XPM(&button[button_id], pfl_down_xpm, TRUE);
		button[button_id].function = pfl_cb;

		button_id = MUTE + count;
		button[button_id].x = 5;
		button[button_id].y = screen[count].position + 22;
		translate_XPM(&button[button_id], mute_up_xpm, FALSE);
		translate_XPM(&button[button_id], mute_down_xpm, TRUE);
		button[button_id].function = mute_cb;

		button_id = EDIT + count;
		button[button_id].x = 5;
		button[button_id].y = screen[count].position + 42;
		translate_XPM(&button[button_id], edit_up_xpm, FALSE);
		translate_XPM(&button[button_id], edit_down_xpm, TRUE);
		button[button_id].function = edit_cb;

		button_id = SWITCH + count;
		button[button_id].x = 5;
		button[button_id].y = screen[count].position + 62;
		translate_XPM(&button[button_id], switch_up_xpm, FALSE);
		translate_XPM(&button[button_id], switch_down_xpm, TRUE);
		button[button_id].function = switch_cb;

		button_id = REC + count;
		button[button_id].x = 5;
		button[button_id].y = screen[count].position + 82;
		translate_XPM(&button[button_id], rec_up_xpm, FALSE);
		translate_XPM(&button[button_id], rec_down_xpm, TRUE);
		button[button_id].function = rec_cb;
	}

	/* Show buttons */
	for (count = 0; count < PFL; count++)
		display_button(&button[count], FALSE);
}
