/*************************************************************************
    callback.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/signal.h>
#include "multitrack.h"

#include "images/ok_down.xpm"
#include "images/ok_up.xpm"
#include "images/cancel_down.xpm"
#include "images/cancel_up.xpm"


static int button_to_track(int button_pressed)
{
	button_pressed -= PFL;
	while (button_pressed >= nr_tracks)
		button_pressed -= nr_tracks;
	return button_pressed;
}


void pfl_cb(int button_pressed)
{
	int track_id = button_to_track(button_pressed);

	if (track[track_id].pfl)
		track[track_id].pfl = FALSE;
	else
		track[track_id].pfl = TRUE;
	wait_for_mouse();
}


void mute_cb(int button_pressed)
{
	int track_id = button_to_track(button_pressed);

	if (track[track_id].mute)
		track[track_id].mute = FALSE;
	else
		track[track_id].mute = TRUE;
	wait_for_mouse();
}


void edit_cb(int button_pressed)
{
	int track_id = button_to_track(button_pressed);

	if (edit_track == track_id)
		edit_track = -1;
	else {
		if (edit_track != -1)
			show_button(&button[EDIT + edit_track], FALSE);
		show_button(&button[EDIT + track_id], TRUE);
		edit_track = track_id;
	}
	if (select_begin != select_end) {
		select_region(screen_min, select_begin - 1, FALSE);
		select_region(select_begin, select_end, TRUE);
		select_region(select_end + 1, screen_max, FALSE);
	} else
		select_region(screen_min, screen_max, FALSE);
	wait_for_mouse();
}


void rec_cb(int button_pressed)
{
	int track_id = button_to_track(button_pressed);

	if (track[track_id].rec) {
		track[track_id].rec = FALSE;
		if (select_begin != select_end)
			select_region(select_begin, select_end, TRUE);
	} else if (tracks_selected() < 2) {
		track[track_id].rec = TRUE;
		if (select_begin != select_end)
			select_region(select_begin, select_end, TRUE);
	} else
		show_button(&button[REC + track_id], FALSE);
	wait_for_mouse();
}


static void buffer_volume_cb(int slider_pressed)
{
	int track_id = slider_pressed;

	while (track_id >= nr_tracks)
		track_id -= nr_tracks;

	bigbuf_info->volume = slider[track_id + VOL_SLIDE].position /
	    (float) (slider[track_id + VOL_SLIDE].length) * 64;
}


void switch_cb(int button_pressed)
{
	int track_id = button_to_track(button_pressed), old_track = -1;

	if (bigbuf_info->show_buffer == track_id) {
		show_button(&button[SWITCH + track_id], FALSE);
		slider[VOL_SLIDE + track_id].function = volume_cb;
		bigbuf_info->show_buffer = -1;
		show_track_window(track_id);
	} else {
		if (bigbuf_info->show_buffer != -1)
			old_track = bigbuf_info->show_buffer;
		bigbuf_info->show_buffer = track_id;
		slider[VOL_SLIDE + track_id].function = buffer_volume_cb;
		show_track_window(track_id);
		if (old_track != -1) {
			show_button(&button[SWITCH + old_track], FALSE);
			slider[VOL_SLIDE + old_track].function = volume_cb;
			show_track_window(old_track);
		}
	}
	delete_mouse();
	show_tracks();
	if (select_begin != select_end)
		select_region(select_begin, select_end, TRUE);
	show_mouse();
}


void volume_cb(int slider_pressed)
{
	float volume, left_right;
	int track_id = slider_pressed;

	while (track_id >= nr_tracks)
		track_id -= nr_tracks;
	volume = slider[track_id + VOL_SLIDE].position /
	    (float) (slider[track_id + VOL_SLIDE].length) * 64;
	left_right = (slider[track_id + LR_SLIDE].position - slider[track_id + LR_SLIDE].length / 2) /
	    (float) (slider[track_id + LR_SLIDE].length) * 2;
	track[track_id].volume_l = (int) volume;
	track[track_id].volume_r = (int) volume;
	if (left_right > 0)
		track[track_id].volume_r = (int) ((1 - left_right) * volume);
	if (left_right < 0)
		track[track_id].volume_l = (int) ((1 + left_right) * volume);

}


void volume_effect_cb(int slider_pressed)
{
	int track_id = slider_pressed;

	while (track_id >= nr_tracks)
		track_id -= nr_tracks;

	track[track_id].volume_effect = slider[slider_pressed].position;
}


void rew_cb(int data)
{
	info->rew = TRUE;
	wait_for_mouse();
	info->rew = FALSE;
	show_button(&button[REW], FALSE);
}


void stop_cb(int data)
{
	wait_for_mouse();
	if (pid_play)
		kill(pid_play, SIGUSR2);	/* stop playing */
	if (pid_record)
		kill(pid_record, SIGUSR2);	/* stop recording */
	show_button(&button[STOP], FALSE);
}


void ff_cb(int data)
{
	info->ff = TRUE;
	wait_for_mouse();
	info->ff = FALSE;
	show_button(&button[FF], FALSE);
}


void load_cb(int data)
{
	int count;

	wait_for_mouse();
	load_mix_settings();
	for (count = 0; count < nr_tracks; count++)
		show_track_sliders(count);
	show_button(&button[LOAD], FALSE);
}


void save_cb(int data)
{
	wait_for_mouse();
	save_mix_settings();
	show_button(&button[SAVE], FALSE);
}


void send_cb(int data)
{
	int count;

	wait_for_mouse();
	info->effect_send = button[SEND].pressed;
	for (count = 0; count < nr_tracks; count++)
		show_track_window(count);
}


void zoomin_cb(int data)
{
	wait_for_mouse();
	zoom_in();
	show_button(&button[ZOOMIN], FALSE);
}


void zoomout_cb(int data)
{
	wait_for_mouse();
	zoom_out();
	show_button(&button[ZOOMOUT], FALSE);
}


void zoom_cb(int data)
{
	wait_for_mouse();
	zoom();
	show_button(&button[ZOOM], FALSE);
}


static void determine_begin_end(void)
{
	if (select_begin == select_end) {
		info->play_begin = track_begin;
		info->play_end = track_end;
	} else {
		info->play_begin = screen_to_sample(select_begin);
		info->play_end = screen_to_sample(select_end);
	}
	info->time_count = info->play_begin;
	show_play_cursor(TRUE);
}


static void winfunction()
{
}


static int show_question(void)
{
	char hulp[80];
	int count;
	Button winbutton[2];
	Window win;
	Window_text text[2];

	win.x = 290;
	win.y = 235;
	win.width = 130;
	win.height = 60;
	win.function = winfunction;

	winbutton[0].x = win.x + 80;
	winbutton[0].y = win.y + 40;
	translate_XPM(&winbutton[0], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[0], ok_down_xpm, TRUE);
	winbutton[0].pressed = FALSE;
	winbutton[0].function = quit_cb;

	winbutton[1].x = win.x + 5;
	winbutton[1].y = win.y + 40;
	translate_XPM(&winbutton[1], cancel_up_xpm, FALSE);
	translate_XPM(&winbutton[1], cancel_down_xpm, TRUE);
	winbutton[1].pressed = FALSE;
	winbutton[1].function = quit_cb;

	text[0].x = win.x + 10;
	text[0].y = win.y + 4;
	sprintf(text[0].string, "Record track:");

	text[1].x = win.x + 10;
	text[1].y = win.y + 20;
	text[1].string[0] = '\0';
	for (count = 0; count < nr_tracks; count++)
		if (track[count].rec) {
			sprintf(hulp, "%d ", count);
			strcat(text[1].string, hulp);
		}
	return do_window(&win, winbutton, 2, NULL, 0, text, 2, 0, 1);
}


void record_cb(int data)
{
	if (tracks_selected() > 0 && !info->playing) {
		if (show_question()) {
			determine_begin_end();
			if (pid_record)
				kill(pid_record, SIGUSR1);	/* start recording */
			usleep(300000);		/* wait 0.3 sec */
			if (pid_play && soundcard[1].ok) {
				show_button(&button[PLAY], TRUE);
				kill(pid_play, SIGUSR1);	/* start playing */
			}
		} else
			show_button(&button[RECORD], FALSE);
	} else
		show_button(&button[RECORD], FALSE);
}


void play_cb(int data)
{
	determine_begin_end();
	if (pid_play)
		kill(pid_play, SIGUSR1);	/* start playing */
}


void mixer0_cb(int data)
{
	if (soundcard[0].ok) {
		if (soundcard[1].ok)
			show_mixer(0, FALSE);
		else if (!info->playing && !info->recording)
			show_mixer(0, TRUE);
	}
	show_button(&button[MIXER0], FALSE);
}


void mixer1_cb(int data)
{
	if (soundcard[1].ok) {
		if (!info->recording && !(info->playing && info->effect_send))
			show_mixer(1, TRUE);
		else 
			show_mixer(1, FALSE);
	}
	show_button(&button[MIXER1], FALSE);
}


void cut_cb(int data)
{
	wait_for_mouse();
	cut_menu();
	show_button(&button[CUT], FALSE);
}


void paste_cb(int data)
{
	wait_for_mouse();
	paste_menu();
	show_button(&button[PASTE], FALSE);
}


void copy_cb(int data)
{
	wait_for_mouse();
	copy_menu();
	show_button(&button[COPY], FALSE);
}


void amplify_cb(int data)
{
	wait_for_mouse();
	amplify_menu();
	show_button(&button[AMPLIFY], FALSE);
}


void quit_cb(int data)
{
	wait_for_mouse();
}
