/*************************************************************************
    copy.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include "multitrack.h"

#include "images/check_up.xpm"
#include "images/check_down.xpm"
#include "images/ok_up.xpm"
#include "images/ok_down.xpm"
#include "images/cancel_up.xpm"
#include "images/cancel_down.xpm"


static int check_overload(int copy_track, int paste_track, float rel_volume)
/* Check if result for overlap paste does not exceed maximum or minimum
   value */
{
	int copy_fd, paste_fd, overload = FALSE;
	long begin = screen_to_sample(select_begin), position;
	long end = screen_to_sample(select_end);
	char filename[80], copy_buffer[2], paste_buffer[2];

	sprintf(filename, "display%d.dat", copy_track);
	copy_fd = open(filename, O_RDONLY);
	if (copy_fd == -1)
		error("Cannot open display-file.");
	sprintf(filename, "display%d.dat", paste_track);
	paste_fd = open(filename, O_RDONLY);
	if (paste_fd == -1)
		error("Cannot open display-file.");

	lseek(copy_fd, (begin / 512) * 2, SEEK_SET);
	lseek(paste_fd, (begin / 512) * 2, SEEK_SET);
	position = begin;
	do {
		read(copy_fd, copy_buffer, 2);
		read(paste_fd, paste_buffer, 2);
		if ((copy_buffer[0] * rel_volume + paste_buffer[0] < -127) ||
		    (copy_buffer[1] * rel_volume + paste_buffer[1] > 128))
			overload = TRUE;
		position += 512;
	}
	while (!overload && position < end);

	return overload;
}


static void copy_track_to_track(int paste_track, int overlap)
{
	int output_fd, input_fd, input2_fd, slicelen;
	int max_slicelen = 16384, i, ok;
	short int *i_data, *o_data;
	char filename[80], hulp[80];
	long position, begin = screen_to_sample(select_begin);
	long end = screen_to_sample(select_end);
	float rel_volume;

	sprintf(filename, "track%d.wav", paste_track);
	output_fd = open(filename, O_WRONLY);
	if (output_fd == -1)
		error("Cannot open track for writing.");

	sprintf(filename, "track%d.wav", edit_track);
	input_fd = open(filename, O_RDONLY);
	if (input_fd == -1)
		error("Cannot open track for reading.");

	sprintf(filename, "track%d.wav", paste_track);
	input2_fd = open(filename, O_RDONLY);
	if (input2_fd == -1)
		error("Cannot open track for reading.");

	i_data = (short int *) malloc(sizeof(short int) * max_slicelen);
	o_data = (short int *) malloc(sizeof(short int) * max_slicelen);
	if (i_data == NULL || o_data == NULL)
		error("Unable to allocate memory.");

	lseek(output_fd, begin * 2, SEEK_SET);
	lseek(input_fd, begin * 2, SEEK_SET);
	lseek(input2_fd, begin * 2, SEEK_SET);

	rel_volume = slider[VOL_SLIDE + edit_track].position /
	    (float) slider[VOL_SLIDE + paste_track].position;

	ok = TRUE;
	if (overlap)
		if (check_overload(edit_track, paste_track, rel_volume)) {
			overload_menu("copy");
			ok = FALSE;
		}
	if (ok) {
		sprintf(hulp, "Copy track %d to track %d.", edit_track, paste_track);
		init_progress(hulp);

		position = 0;
		do {
			slicelen = end - begin + 1 - position;
			if (slicelen > max_slicelen)
				slicelen = max_slicelen;

			read(input_fd, (char *) i_data, slicelen * 2);
			if (overlap) {
				read(input2_fd, (char *) o_data, slicelen * 2);
				for (i = 0; i < slicelen; i++)
					o_data[i] += (short int) (i_data[i] * rel_volume);
				write(output_fd, (char *) o_data, slicelen * 2);
			} else
				write(output_fd, (char *) i_data, slicelen * 2);

			position += slicelen;
			show_progress(position / (float) (end - begin + 1));
		}
		while (slicelen == max_slicelen);

		delete_progress();
	}
	close(output_fd);
	close(input_fd);
	if (overlap)
		close(input2_fd);
	if (ok)
		make_display_file(paste_track, begin, end);
}



static Button winbutton[6];


static void track_cb(int button_id)
{
	show_button(&winbutton[0], FALSE);
	show_button(&winbutton[1], FALSE);
	show_button(&winbutton[2], FALSE);

	show_button(&winbutton[button_id], TRUE);
}


static void winfunction(void)
{
}


void copy_menu2(void)
{
	Window_text text[5];
	Window win;
	int i, count, copy_track[4];

	win.x = 270;
	win.y = 225;
	win.width = 200;
	win.height = 70;
	win.function = winfunction;

	winbutton[0].x = win.x + 5;
	winbutton[0].y = win.y + 15;
	translate_XPM(&winbutton[0], check_up_xpm, FALSE);
	translate_XPM(&winbutton[0], check_down_xpm, TRUE);
	winbutton[0].pressed = TRUE;
	winbutton[0].active = TRUE;
	winbutton[0].function = track_cb;

	winbutton[1].x = win.x + 45;
	winbutton[1].y = win.y + 15;
	translate_XPM(&winbutton[1], check_up_xpm, FALSE);
	translate_XPM(&winbutton[1], check_down_xpm, TRUE);
	winbutton[1].pressed = FALSE;
	winbutton[1].active = TRUE;
	winbutton[1].function = track_cb;

	winbutton[2].x = win.x + 85;
	winbutton[2].y = win.y + 15;
	translate_XPM(&winbutton[2], check_up_xpm, FALSE);
	translate_XPM(&winbutton[2], check_down_xpm, TRUE);
	winbutton[2].pressed = FALSE;
	winbutton[2].active = TRUE;
	winbutton[2].function = track_cb;

	winbutton[3].x = win.x + 5;
	winbutton[3].y = win.y + 45;
	translate_XPM(&winbutton[3], check_up_xpm, FALSE);
	translate_XPM(&winbutton[3], check_down_xpm, TRUE);
	winbutton[3].pressed = FALSE;
	winbutton[3].active = TRUE;
	winbutton[3].function = quit_cb;

	winbutton[4].x = win.x + win.width - 50;
	winbutton[4].y = win.y + win.height - 20;
	translate_XPM(&winbutton[4], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[4], ok_down_xpm, TRUE);
	winbutton[4].pressed = FALSE;
	winbutton[4].active = TRUE;
	winbutton[4].function = quit_cb;

	winbutton[5].x = win.x + win.width - 50;
	winbutton[5].y = win.y + win.height - 40;
	translate_XPM(&winbutton[5], cancel_up_xpm, FALSE);
	translate_XPM(&winbutton[5], cancel_down_xpm, TRUE);
	winbutton[5].pressed = FALSE;
	winbutton[5].active = TRUE;
	winbutton[5].function = quit_cb;

	count = 0;
	for (i = 0; i < nr_tracks; i++)
		if (i != edit_track) {
			text[count].x = win.x + 25 + 40 * count;
			text[count].y = win.y + 19;
			sprintf(text[count].string, "%d", i);
			copy_track[count] = i;
			count++;
		}
	text[3].x = win.x + 25;
	text[3].y = win.y + 49;
	sprintf(text[3].string, "Overlap copy");
	text[4].x = win.x + 5;
	text[4].y = win.y + 5;
	sprintf(text[4].string, "Copy track %d to:", edit_track);

	if (do_window(&win, winbutton, 6, NULL, 0, text, 5, 4, 5)) {
		count = 0;
		for (i = 0; i < 3; i++)
			if (winbutton[i].pressed)
				count = copy_track[i];
		copy_track_to_track(count, winbutton[3].pressed);
	}
}
