/*************************************************************************
    gui.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/stat.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"

void show_play_cursor(int first_time)
{
	int count;

	play_position = (int) ((float) (info->time_count - track_begin) / (track_end - track_begin) *
			       screen_length + screen_min);
	if (play_position < screen_min)
		play_position = screen_min;
	if (play_position > screen_max)
		play_position = screen_max;

	if (play_position != old_play_position && !first_time) {
		if (play_position < mouse_x + 12 && play_position > mouse_x - 5)
			delete_mouse();
		for (count = 0; count < nr_tracks; count++)
			gl_line(play_position, screen[count].position, play_position,
				screen[count].position + screen[count].height, LIGHTBLUE);
		display_tracks(old_play_position, old_play_position);
		if (play_position < mouse_x + 12 && play_position > mouse_x - 5)
			show_mouse();
	}
	old_play_position = play_position;
}


static void dummyfunction(void)
{
}


static int button_action(int status)
{
	int quit = FALSE;

	switch (status) {
	case MOUSE_LEFTBUTTON:
		if (track_window())
			select_track();
		else if (scrollbar_window())
			move_scrollbar();
		else {
			if (press_button(button, nr_buttons) == QUIT)
				return TRUE;
			press_slider(slider, nr_sliders, dummyfunction);
		}
		break;
	case MOUSE_MIDDLEBUTTON:
		if (track_window() && !(info->playing)) {
			if (bigbuf_info->show_buffer != -1) {
				bigbuf_info->position = screen_to_sample(mouse_x);
				delete_mouse();
				show_buffer(bigbuf_info->show_buffer);
				display_tracks(screen_min, screen_max);
				show_mouse();
			}
		}
		break;
	case MOUSE_RIGHTBUTTON:
		if (track_window() && !mouse_moved && !(info->playing)) {
			delete_mouse();
			if (select_begin == select_end) {
				select_begin = screen_min;
				select_end = screen_max;
				select_region(select_begin, select_end, TRUE);
			} else {
				select_region(select_begin, select_end, FALSE);
				select_begin = screen_min;
				select_end = screen_min;
			}
			show_mouse();
		}
		break;
	default:
		break;
	}
	return quit;
}


void save_guiscreen(void)
{
	unsigned char *gui_screen;
	FILE *fd;
	int x, y, i;
	int color[11] =
	{BLACK, BLUE, DARKGREY, DARKGREEN, LIGHTBLUE, GREY,
	 GREEN, RED, PURPLE, YELLOW, WHITE};
	char color_s[11][20] =
	{"black", "blue", "darkgrey", "darkgreen", "lightblue",
	 "grey", "green", "red", "purple", "yellow", "white"};

	gui_screen = (char *) malloc(640 * 480);
	if (gui_screen == NULL)
		error("Cannot allocate memory.");

	gl_getbox(0, 0, 640, 480, gui_screen);

	fd = fopen("screen.xpm", "w");
	if (fd == NULL)
		error("Cannot open screen.dat");

	fprintf(fd, "/* XPM */\n");
	fprintf(fd, "static char * save_up_xpm[] = {\n");
	fprintf(fd, "\"640 480 11 1\",\n");
	for (i = 0; i < 11; i++)
		if (i != 10)
			fprintf(fd, "\"%d   c %s\",\n", i, color_s[i]);
		else
			fprintf(fd, "\"a   c %s\",\n", color_s[i]);

	for (y = 0; y < 480; y++) {
		fprintf(fd, "\"");
		for (x = 0; x < 640; x++) {
			i = 0;
			while (gui_screen[x + 640 * y] != color[i] && i < 11)
				i++;
			if (i != 11) {
				if (i == 10)
					fprintf(fd, "a");
				else
					fprintf(fd, "%d", i);
			} else
				fprintf(fd, "0");
		}
		fprintf(fd, "\",\n");
	}

	free(gui_screen);
	fclose(fd);
}


void gui(void)
{
	char ch, *save_infoscreen;
	int quit = FALSE, count, time_count = 0, show_info = FALSE;
	int button_id;

	open_screen();
	init_tracks();
	init_buttons();
	init_sliders();
	init_mouse();
	for (count = 0; count < nr_tracks; count++)
		show_track_window(count);
	show_tracks();
	show_scrollbar();
	move_mouse();

	save_infoscreen = (char *) malloc(13 * (40 * 8 + 3) );
	if (save_infoscreen == NULL)
		error("Cannot allocate save_infoscreen.");

	while (!quit) {
		usleep(1000);
		button_id = button_region(button, nr_buttons);
		if (button_id != -1) {
			time_count++;
			if (time_count > 50 && !show_info && strlen(button[button_id].info) > 0) {
				show_info = TRUE;
				delete_mouse();
				gl_getbox(button[button_id].x, button[button_id].y +
					  button[button_id].height + 4, strlen(button[button_id].info) * 8 + 3,
					  12, save_infoscreen);
				gl_fillbox(button[button_id].x, button[button_id].y +
					   button[button_id].height + 4, strlen(button[button_id].info) * 8 + 3,
					   12, DARKGREY);
				gl_fillbox(button[button_id].x + 1, button[button_id].y +
					   button[button_id].height + 5, strlen(button[button_id].info) * 8 + 1,
					   10, YELLOW);
				gl_colorfont(8, 8, DARKGREY, font);
				gl_write(button[button_id].x + 2, button[button_id].y +
					 button[button_id].height + 6, button[button_id].info);
				gl_colorfont(8, 8, WHITE, font);
				show_mouse();
			}
		} else
			time_count = 0;

		if (mouse_update()) {
			if (show_info) {
				show_info = FALSE;
				delete_mouse();
				gl_putbox(button[button_id].x, button[button_id].y +
					  button[button_id].height + 4, strlen(button[button_id].info) * 8 + 3,
					  12, save_infoscreen);
				show_mouse();
			}
			time_count = 0;
			move_mouse();
			quit = button_action(mouse_getbutton());
		}
		if (info->playing || info->recording)	/* move play cursor */
			show_play_cursor(FALSE);
		if (info->finished_playing) {	/* reset play button, clear play cursor */
			info->finished_playing = FALSE;
			button[PLAY].active = TRUE;
			show_button(&button[PLAY], FALSE);
			display_tracks(old_play_position, old_play_position);
		}
		if (info->finished_recording) {
			info->finished_recording = FALSE;
			button[RECORD].active = TRUE;
			show_button(&button[RECORD], FALSE);
			for (count = 0; count < nr_tracks; count++)
				if (track[count].rec)
					make_display_file(count, info->play_begin, info->play_end);
			show_tracks();
			if (select_end != select_begin) {
				select_region(select_begin, select_end, TRUE);
				display_tracks(select_begin, select_end);
			}
		}
		if ((ch = vga_getkey()))
			switch (ch) {
			case 's':
				save_guiscreen();
				break;
			case 'q':
				quit = TRUE;
				break;
			case 'z':
				zoom_in();
				break;
			case 'x':
				zoom_out();
				break;
			default:
				break;
			}
	}
}
