/*************************************************************************
    io.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include "multitrack.h"


void debug(char *s)
{
	if (DEBUG)
		fprintf(stderr, "Debug: %s\n", s);
}


void help(void)
{
	fprintf(stderr, "\nThis is MultiTrack v1.0b\n");
	fprintf(stderr, "Copyright 1996 by B. Nagels\n");
	fprintf(stderr, "E-mail: boris@rulhm1.LeidenUniv.nl\n\n");
	exit(0);
}


void init_soundcards(void)
{
	int fd[2], card;
	char hulp[80];

	soundcard[0].ok = FALSE;
	soundcard[1].ok = FALSE;
	soundcard[0].dual_dma = FALSE;
	soundcard[1].dual_dma = FALSE;

	sprintf(hulp, "/dev/dsp%d", soundcard[0].device);
	fd[0] = open(hulp, O_RDONLY);
	if (fd[0] != -1)		/* soundcard 0 ok */
		soundcard[0].ok = TRUE;

	sprintf(hulp, "/dev/dsp%d", soundcard[1].device);
	fd[1] = open(hulp, O_RDONLY);
	if (fd[1] != -1)		/* soundcard 1 ok */
		soundcard[1].ok = TRUE;
	close(fd[0]);
	close(fd[1]);
	
	if (soundcard[1].ok && !soundcard[0].ok) {	/* change order */
		card = soundcard[0].device;
		soundcard[0].device = soundcard[1].device;
		soundcard[1].device = card;
		soundcard[0].ok = TRUE;
		soundcard[1].ok = FALSE;
	}
	
	if (soundcard[0].ok && !soundcard[1].ok) {	/* check DUAL-DMA */
		sprintf(hulp, "/dev/dsp%d", soundcard[0].device);
		fd[0] = open(hulp, O_RDONLY);
		sprintf(hulp, "/dev/dsp%d", soundcard[1].device);
		fd[1] = open(hulp, O_WRONLY);
		if (fd[1] != -1)
			soundcard[0].dual_dma = TRUE;
		close(fd[0]);
		close(fd[1]);
	}
}


void open_audio(int *audio_fd, int card, int mode, int dsp_stereo)
{
	int dsp_speed, dsp_samplesize = AFMT_S16_LE;
	int arg = 0xFFFF000A;	/* buffers of 1kb */
	char hulp[80];
	int abuf_size;

	/* open audio of soundcard 0 or 1 */
	sprintf(hulp, "/dev/dsp%d", soundcard[card].device);
	dsp_speed = soundcard[card].samplerate;
	if ((*audio_fd = open(hulp, mode)) == -1)
		error("Error opening audio device.");

	if (mode == O_WRONLY) {		/* the output soundcard */
		if (ioctl(*audio_fd, SNDCTL_DSP_SETFRAGMENT, &arg))
			error("Error setting fragment");
	}
	if (ioctl(*audio_fd, SNDCTL_DSP_SYNC, NULL) < 0)
		error("Error syncing audio device.");
	if (ioctl(*audio_fd, SNDCTL_DSP_STEREO, &dsp_stereo) == -1)
		error("Error setting stereo/mono.");
	if (ioctl(*audio_fd, SNDCTL_DSP_SAMPLESIZE, &dsp_samplesize) == -1)
		error("Error setting 16-bit mode.");
	if (ioctl(*audio_fd, SNDCTL_DSP_SPEED, &dsp_speed) == -1)
		error("Error setting frequency.");
	ioctl(*audio_fd, SNDCTL_DSP_GETBLKSIZE, &abuf_size);

	if (DEBUG);
/*              fprintf(stderr, "Audio buffer size: %d bytes.\n", abuf_size); */
}


void load_mix_settings(void)
{
	FILE *fd;
	int count, tmp1, tmp2, tmp3;

	fd = fopen("multitrack.mix", "r");
	if (fd != NULL) {
		for (count = 0; count < nr_tracks; count++) {
			fscanf(fd, "%d   %d   %d", &tmp1, &tmp2, &tmp3);
			track[count].volume_l = tmp1;
			track[count].volume_r = tmp2;
			track[count].volume_effect = tmp3;
		}
	}
	fclose(fd);
}


void save_mix_settings(void)
{
	FILE *fd;
	int count;

	fd = fopen("multitrack.mix", "w");
	if (fd != NULL) {
		for (count = 0; count < nr_tracks; count++)
			fprintf(fd, "%2d   %2d   %2d\n", track[count].volume_l,
				track[count].volume_r, track[count].volume_effect);
	}
	fclose(fd);

}


static int read_line(FILE * fd, char *buffer)
{
	int stop = FALSE, end_of_file = FALSE;

	do {
		if (fgets(buffer, 132, fd) == NULL)
			end_of_file = TRUE;
		if (buffer[0] != '#' && (buffer[0] == 'i' || buffer[0] == 'o'))
			stop = TRUE;
	}
	while (!stop && !end_of_file);

	return end_of_file;
}


void load_RC(void)
{
	FILE *fd;
	char buffer[132], filename[80];
	int stop, card;

	/* set default values */
	soundcard[0].device = 0;	/* the output device */
	soundcard[1].device = 1;	/* the input device */
	soundcard[0].samplerate = 22050;
	soundcard[1].samplerate = 22050;
	soundcard[0].buffersize = 256;
	soundcard[1].buffersize = 1024;

	if (getenv("HOME") == NULL)
		error("Cannot find HOME environment variable.");
	else
		sprintf(filename, "%s/.multitrackrc", getenv("HOME"));
	fd = fopen(filename, "r");
	if (fd != NULL) {
		fprintf(stderr, "Parsing %s\n", filename);
		stop = FALSE;
		do {
			stop = read_line(fd, buffer);
			if (!stop) {
				if (buffer[0] == 'o')
					card = 0;
				else
					card = 1;
				sscanf(buffer + 1, "_card = %d", &soundcard[card].device);
				sscanf(buffer + 1, "_samplerate = %d", &soundcard[card].samplerate);
				sscanf(buffer + 1, "_buffersize = %d", &soundcard[card].buffersize);
			}
		}
		while (!stop);
		fclose(fd);
	} else
		fprintf(stderr, "Cannot find %s. Using defaults.\n", filename);

	if (0)
		for (card = 0; card < 2; card++) {
			if (card == 0)
				fprintf(stderr, "Output:\n");
			else
				fprintf(stderr, "\nInput:\n");
			fprintf(stderr, "/dev/dsp%d\n", soundcard[card].device);
			fprintf(stderr, "%d Hz\n", soundcard[card].samplerate);
			fprintf(stderr, "%d bytes buffer\n", soundcard[card].buffersize);
		}
}
