/*************************************************************************
    mixer.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include "multitrack.h"


void write_mixer(int mixer_fd, int channel, int level)
{
	level |= (level << 8);
	if (ioctl(mixer_fd, MIXER_WRITE(channel), &level) == -1)
		fprintf(stderr, "Error writing to mixer device: %d.\n", MIXER_WRITE(channel));
}


void init_mixers(void)
{
	int i, bit, count;
	int mixer_fd[2], dev_mask;
	char hulp[20];

	/* open the mixers for both soundcards */

	for (i = 0; i < 2; i++) {
		sprintf(hulp, "/dev/mixer%d", soundcard[i].device);
		mixer_fd[i] = open(hulp, O_RDWR);
		if (soundcard[i].ok) {
			ioctl(mixer_fd[i], SOUND_MIXER_READ_DEVMASK, &dev_mask);
			count = 0;
			for (bit = 0; bit < 20; bit++)
				if (dev_mask & (1 << bit)) {
					soundcard[i].mixer_lines[count] = bit;
					count++;
				}
			soundcard[i].nr_mixer_lines = count;
		}
	}

	if (soundcard[0].ok)
		close(mixer_fd[0]);
	if (soundcard[1].ok)
		close(mixer_fd[1]);
}


void mute_mixer(int mixer_fd, int mute)
{
	int level = mute ? 0 : 100;

	write_mixer(mixer_fd, SOUND_MIXER_PCM, level);
}


#include "images/ok_up.xpm"
#include "images/ok_down.xpm"
#include "images/check_up.xpm"
#include "images/check_down.xpm"
#include "images/load_up.xpm"
#include "images/load_down.xpm"
#include "images/save_up.xpm"
#include "images/save_down.xpm"

static int audio_fd, mixer_fd;
static Slider winslider[20];
static Button winbutton[20];
static Window win;
static int card, nr_winbuttons, recline[20];
static fragment = 512;
static char *black, *bar;
static short int *data;

static void init_vu_meter(void)
{
	int i, color, led;

	/* allocate data arrays and initialize */
	data = (short int *) malloc(sizeof(short int) * fragment * 2);
	black = (char *) malloc(sizeof(char) * 210);
	bar = (char *) malloc(sizeof(char) * 210);
	if (data == NULL || black == NULL || bar == NULL)
		error("Cannot allocate data-array.");

	for (led = 0; led < 30; led++)
		for (i = 0; i < 7; i++) {
			black[i + led * 7] = BLACK;
			color = GREEN;
			if (led > 25)
				color = RED;
			if (i < 5)
				bar[i + led * 7] = color;
			else
				bar[i + led * 7] = BLACK;
		}
}


static void show_vu_meter(void)
{
	int i, line, x = win.x + 20, y = win.y + 5;
	int max_left, max_right;

	read(audio_fd, (char *) data, fragment * 2 * 2);	/* stereo 16-bit */
	max_left = abs(data[0]);
	max_right = abs(data[1]);
	for (i = 0; i < fragment / 2; i++) {
		if (abs(data[i * 2]) > max_left)
			max_left = abs(data[i * 2]);
		if (abs(data[i * 2 + 1]) > max_right)
			max_right = abs(data[i * 2 + 1]);
	}
	max_left = (int) (max_left / 32768.0 * 30);
	max_right = (int) (max_right / 32768.0 * 30);

	if (max_left > 30)
		max_left = 30;
	if (max_right > 30)
		max_right = 30;

	for (line = 0; line < 4; line++) {
		vga_drawscansegment(black, x, y + line, 210);
		vga_drawscansegment(bar, x, y + line, max_left * 7);
		vga_drawscansegment(black, x, y + line + 10, 210);
		vga_drawscansegment(bar, x, y + line + 10, max_right * 7);
	}
}


static void mixerslider_cb(int slider_id)
{
	char hulp[10];
	int x, y;

	write_mixer(mixer_fd, soundcard[card].mixer_lines[slider_id], winslider[slider_id].position);
	sprintf(hulp, "%d", winslider[slider_id].position);
	x = winslider[slider_id].x - strlen(hulp) * 4 + 8;
	y = winslider[slider_id].y - 15;
	gl_fillbox(winslider[slider_id].x - 4, y, 24, 7, DARKGREY);
	gl_write(x, y, hulp);
}


static void mixerbutton_cb(int button_id)
{
	int i;

	for (i = 0; i < nr_winbuttons - 1; i++)
		show_button(&winbutton[i], FALSE);

	ioctl(mixer_fd, SOUND_MIXER_WRITE_RECSRC, &recline[button_id]);
	show_button(&winbutton[button_id], TRUE);
}


static void load_settings_cb(int button_id)
{
	FILE *fd;
	char filename[80];
	int count, vol;

	sprintf(filename, "mixer%d.settings", soundcard[card].device);
	fd = fopen(filename, "r");
	if (fd != NULL) {
		for (count = 0; count < soundcard[card].nr_mixer_lines; count++) {
			fscanf(fd, "%d", &vol);
			winslider[count].position = vol;
			show_slider(winslider[count], FALSE);
			mixerslider_cb(count);
		}
	}
	fclose(fd);
	wait_for_mouse();
	show_button(&winbutton[button_id], FALSE);
}


static void save_settings_cb(int button_id)
{
	FILE *fd;
	char filename[80];
	int count, vol;

	sprintf(filename, "mixer%d.settings", soundcard[card].device);
	fd = fopen(filename, "w");
	if (fd == NULL)
		error("Cannot open mixerfile for writing");

	for (count = 0; count < soundcard[card].nr_mixer_lines; count++) {
		ioctl(mixer_fd, MIXER_READ(soundcard[card].mixer_lines[count]), &vol);
		fprintf(fd, "%d\n", (vol >> 8));
	}

	fclose(fd);
	wait_for_mouse();
	show_button(&winbutton[button_id], FALSE);
}


static void init_window(void)
{
	int vol, count, recmask, source, bit;

	nr_winbuttons = 0;
	ioctl(mixer_fd, SOUND_MIXER_READ_RECMASK, &recmask);
	ioctl(mixer_fd, SOUND_MIXER_READ_RECSRC, &source);
	for (count = 0; count < soundcard[card].nr_mixer_lines; count++) {
		winslider[count].x = win.x + 10 + 45 * count;
		winslider[count].y = win.y + 60;
		winslider[count].active = TRUE;
		winslider[count].length = 100;
		ioctl(mixer_fd, MIXER_READ(soundcard[card].mixer_lines[count]), &vol);
		winslider[count].position = (vol >> 8);
		winslider[count].function = mixerslider_cb;
		bit = 1 << soundcard[card].mixer_lines[count];
		if (recmask & bit) {
			winbutton[nr_winbuttons].x = win.x + 10 + 45 * count;
			winbutton[nr_winbuttons].y = win.y + win.height - 20;
			translate_XPM(&winbutton[nr_winbuttons], check_up_xpm, FALSE);
			translate_XPM(&winbutton[nr_winbuttons], check_down_xpm, TRUE);
			if (source & bit)
				winbutton[nr_winbuttons].pressed = TRUE;
			else
				winbutton[nr_winbuttons].pressed = FALSE;
			winbutton[nr_winbuttons].active = TRUE;
			winbutton[nr_winbuttons].function = mixerbutton_cb;
			recline[nr_winbuttons] = bit;
			nr_winbuttons++;
		}
	}
	winbutton[nr_winbuttons].x = win.x + win.width - 100;
	winbutton[nr_winbuttons].y = win.y + 5;
	translate_XPM(&winbutton[nr_winbuttons], load_up_xpm, FALSE);
	translate_XPM(&winbutton[nr_winbuttons], load_down_xpm, TRUE);
	winbutton[nr_winbuttons].pressed = FALSE;
	winbutton[nr_winbuttons].active = TRUE;
	winbutton[nr_winbuttons].function = load_settings_cb;
	nr_winbuttons++;

	winbutton[nr_winbuttons].x = win.x + win.width - 75;
	winbutton[nr_winbuttons].y = win.y + 5;
	translate_XPM(&winbutton[nr_winbuttons], save_up_xpm, FALSE);
	translate_XPM(&winbutton[nr_winbuttons], save_down_xpm, TRUE);
	winbutton[nr_winbuttons].pressed = FALSE;
	winbutton[nr_winbuttons].active = TRUE;
	winbutton[nr_winbuttons].function = save_settings_cb;
	nr_winbuttons++;

	winbutton[nr_winbuttons].x = win.x + win.width - 49;
	winbutton[nr_winbuttons].y = win.y + 5;
	translate_XPM(&winbutton[nr_winbuttons], ok_up_xpm, FALSE);
	translate_XPM(&winbutton[nr_winbuttons], ok_down_xpm, TRUE);
	winbutton[nr_winbuttons].pressed = FALSE;
	winbutton[nr_winbuttons].active = TRUE;
	winbutton[nr_winbuttons].function = quit_cb;
	nr_winbuttons++;
}


static void winfunction(void)
{
}


void show_mixer(int card_nr, int show_monitor)
{
	int count;
	int nr_winsliders;
	Window_text text[42];
	char *dev_label[] = SOUND_DEVICE_LABELS, hulp[20];

	show_button(&button[MIXER0 + card_nr], TRUE);
	card = card_nr;

	nr_winsliders = soundcard[card].nr_mixer_lines;
	win.width = nr_winsliders * 45 + 5;
	win.x = (640 - win.width) / 2;
	win.y = 170;
	win.height = 190;
	win.function = winfunction;

	sprintf(hulp, "/dev/mixer%d", soundcard[card].device);
	mixer_fd = open(hulp, O_RDWR);

	if (show_monitor) {
		init_vu_meter();
		open_audio(&audio_fd, card, O_RDONLY, TRUE);

		text[nr_winsliders * 2].x = win.x + 10;
		text[nr_winsliders * 2].y = win.y + 3;
		sprintf(text[nr_winsliders * 2].string, "L");
		text[nr_winsliders * 2 + 1].x = win.x + 10;
		text[nr_winsliders * 2 + 1].y = win.y + 13;
		sprintf(text[nr_winsliders * 2 + 1].string, "R");
		win.function = show_vu_meter;
	}
	init_window();

	for (count = 0; count < nr_winsliders; count++) {
		text[count * 2].x = win.x + 8 + 45 * count;
		text[count * 2].y = win.y + 32;
		sprintf(text[count * 2].string, "%s", dev_label[soundcard[card].mixer_lines[count]]);
		sprintf(text[count * 2 + 1].string, "%d", winslider[count].position);
		text[count * 2 + 1].x = winslider[count].x - strlen(text[count * 2 + 1].string) * 4 + 8;
		text[count * 2 + 1].y = winslider[count].y - 15;
	}

	if (info->playing)
		display_tracks(old_play_position, old_play_position);
	do_window(&win, winbutton, nr_winbuttons, winslider, nr_winsliders,
		  text, nr_winsliders * 2 + show_monitor * 2, nr_winbuttons - 1, -2);

	close(mixer_fd);
	if (show_monitor) {
		close(audio_fd);
		free(data);
		free(black);
		free(bar);
	}
}
