/*************************************************************************
    mouse.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"
#include "images/mouse.xbm"	/* kind of bitmap */


void init_mouse(void)
/* init the svgalib mouse */
{
	if (mouse_init("/dev/mouse", vga_getmousetype(), MOUSE_DEFAULTSAMPLERATE))
		error("Could not initialize mouse.");

	mouse_setdefaulteventhandler();
	mouse_setposition(WIDTH / 2, HEIGHT / 2);
	mouse_setxrange(0, WIDTH - 1);
	mouse_setyrange(0, HEIGHT - 1);

	mouse_x = mouse_getx();
	mouse_y = mouse_gety();
	mouse_moved = FALSE;

	/* save screen under initial mouse position */
	save_screen = (char *) malloc(sizeof(char) * (mouse_width + 1) * (mouse_height + 1));
	gl_getbox(mouse_x, mouse_y, mouse_width, mouse_height, save_screen);
}


void wait_for_mouse(void)
{
	int status;

	status = mouse_getbutton();
	while (status == MOUSE_LEFTBUTTON) {
		usleep(10000);
		if (info->playing && !sub_window)
			show_play_cursor(FALSE);
		if (mouse_update())
			status = mouse_getbutton();
	}
}


void show_mouse(void)
{
	int count, color;
	char hulp[80];

	/* show new mouse cursor */
	if (mouse_x < screen_max - mouse_width) {
		gl_getbox(mouse_x, mouse_y, mouse_width, mouse_height, save_screen);
		gl_putboxmask(mouse_x, mouse_y, mouse_width, mouse_height,
			      mouse_bits);
	}
	if (track_window() && !sub_window) {
		for (count = 0; count < nr_tracks; count++) {
			if (mouse_x >= select_begin &&
			    mouse_x <= select_end &&
			    bigbuf_info->show_buffer != count)
				color = BLACK;
			else
				color = RED;
			if (mouse_x <= screen_max)
				gl_line(mouse_x, screen[count].position, mouse_x,
					screen[count].position + screen[count].height, color);
		}
		sprintf(hulp, "%0.3f", screen_to_sample(mouse_x) / 22050.0);
		gl_write(0, 471, hulp);
	}
}


void delete_mouse(void)
{
	/* delete old mouse cursor */
	if (mouse_x < screen_max - mouse_width)
		gl_putbox(mouse_x, mouse_y, mouse_width, mouse_height, save_screen);

	if (track_window() && !sub_window) {
		display_tracks(mouse_x, mouse_x);
		gl_fillbox(0, 471, 60, 7, BLACK);
	}
}


void move_mouse(void)
{
	int old_x = mouse_x, old_y = mouse_y;

	delete_mouse();

	mouse_x = mouse_getx();
	mouse_y = mouse_gety();

	if (mouse_x == old_x && mouse_y == old_y)
		mouse_moved = FALSE;
	else
		mouse_moved = TRUE;

	show_mouse();
}
