/*************************************************************************
    multitrack.h -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#define FALSE	    0
#define TRUE	    1
#define DEBUG       1

#define nr_outputs  	2
#define nr_tracks   	4
#define screen_min	195	/* define a window of 441 pixels = 635-195+1 */
#define screen_max	635	/* (= 2 seconds sampletime) */
#define blocksize	512	/* 512 samples per display line */

/* Define the global buttons */
#define ZOOMIN		0
#define ZOOMOUT		1
#define ZOOM		2
#define LOAD		3
#define SAVE		4
#define SEND		5
#define REW		6
#define STOP		7
#define PLAY		8
#define FF		9 
#define RECORD		10
#define MIXER0		11
#define MIXER1		12
#define CUT		13
#define PASTE		14
#define COPY		15
#define AMPLIFY		16
#define QUIT		17
#define PFL		18
#define MUTE		(PFL + nr_tracks)
#define EDIT		(MUTE + nr_tracks)
#define REC		(EDIT + nr_tracks)
#define SWITCH		(REC + nr_tracks)

#define nr_buttons	(5 * nr_tracks + 18)	/* PFL, mute, rec, edit and switch */
						/* for each track */
#define LR_SLIDE	0
#define VOL_SLIDE	(LR_SLIDE + nr_tracks)
#define EFFECT_SLIDE	(VOL_SLIDE + nr_tracks)
#define nr_sliders	(nr_tracks * 3)

/* Define the colors */
#define BLACK           0
#define BLUE            4
#define DARKGREY	9
#define DARKGREEN	16
#define LIGHTBLUE       31
#define GREY            82
#define GREEN           112
#define RED             128
#define PURPLE		150
#define YELLOW          240
#define WHITE		255

typedef struct Track {
	int pfl;
	int mute;
	int rec;
	int volume_l;
	int volume_r;
	int volume_effect;
} Track;


typedef struct Track_info {
	long time_count;
	long play_begin;
	long play_end;
	int effect_send;
	int playing;
	int finished_playing;
	int recording;
	int finished_recording;
	int ff;
	int rew;
} Track_info;


typedef struct Buffer_info {
	int filled;
	long length;
	long position;
	int volume;
	int show_buffer;
	int reverse;
} Buffer_info;


typedef struct Button {
	int x, y, pressed, active;
	int width, height;
	char *data[2];
	char info[30];
	void (*function)(int parameter);
} Button;


typedef struct Slider {
	int x, y, active;
	int length;
	int position;	/* current position of the slider */
	int middle;
	void (*function)(int parameter);
} Slider;


typedef struct Soundcard {
	int device;
	int ok;
	int dual_dma;
	int samplerate;
	int buffersize;
	int mixer_lines[20];
	int rec_lines[20];
	int nr_mixer_lines;
} Soundcard;


typedef struct Screen {
	int position, height;
	unsigned char *data;
} Screen;


typedef struct Window {
	int x, y;
	int width, height;
	void (*function)(void);
} Window;


typedef struct Window_text {
	int x, y;
	char string[80];
} Window_text;


#include <unistd.h>		/* for pid_t */

/* extern global definitions for screen functions */
extern Button button[nr_buttons];
extern Slider slider[nr_sliders];
extern int screen_length;
extern int edit_track;
extern int mouse_x, mouse_y, mouse_moved, sub_window;
extern int select_begin, select_end;
extern int play_position, old_play_position;
extern char *save_screen;
extern void *font;

/* Global definitions for tracks */
extern Track_info *info;
extern pid_t pid_play, pid_record;
extern Track *track;
extern short int *bigbuf;
extern Buffer_info *bigbuf_info;
extern Screen screen[nr_tracks];
extern long track_begin, track_end, track_length;

/* Global definition for the soundcards */
extern Soundcard soundcard[2];

/* Function in gui.c */
void show_play_cursor(int first_time);
void gui(void);

/* Functions in screen.c */
long screen_to_sample(int screen_x);
void gl_box(int x, int y, int width, int height, int color);
void open_screen(void);
int track_window(void);
int scrollbar_window(void);
void select_region(int begin, int end, int select);
void select_track(void);
void zoom_in(void);
void zoom_out(void);
void zoom(void);
void show_scrollbar(void);
void move_scrollbar(void);

/* Function in main.c */
void error(char *s);

/* Functions in io.c */
void debug(char *s);
void help(void);
void init_soundcards(void);
void open_audio(int *audio_fd, int soundcard, int mode, int dsp_stereo);
void load_RC(void);
void load_mix_settings(void);
void save_mix_settings(void);

/* Functions in mixer.c */
void write_mixer(int mixer_fd, int channel, int level);
void init_mixers(void);
void mute_mixer(int mixer_fd, int mute);
void show_mixer(int card_nr, int show_monitor);

/* Functions in buffer.c */
void cut_menu(void);
void overload_menu(char *sort);
void paste_menu(void);
void copy_menu(void);
void show_buffer(int track_nr);

/* Functions in track.c */
void init_tracks(void);
void open_tracks(void);
void close_tracks(void);
int tracks_selected(void);
void show_tracks(void);
void show_track_window(int track_id);
void max_min(short int *buffer, int number, char *output_buf);
void make_display_file(int track_nr, long begin, long end);
void display_tracks(int begin, int end);

/* Functions in amplify.c */
void amplify_menu(void);

/* Functions in copy.c */
void copy_menu2(void);

/* Function in play.c */
void main_playchild(void);

/* Function in record.c */
void main_recordchild(void);

/* Functions in window.c */
int do_window(Window *win, Button *but, int nr_but, Slider *slid, int nr_slid,
		Window_text *text, int nr_text, int stop_button, int cancel_button);
void init_progress(char *string);
void show_progress(float percent);
void delete_progress(void);

/* Functions in button.c */
int button_region(Button *but, int number_buttons);
int press_button(Button *but, int number_buttons);
void show_button(Button *but, int pressed);
void show_track_buttons(int track_id);
void init_buttons(void);
void translate_XPM(Button *but, char **data, int pressed);

/* Functions in slider.c */
int press_slider(Slider *slid, int number_sliders, void (*function)(void));
void show_slider(Slider slid, int pressed);
void show_track_sliders(int track_id);
void init_sliders(void);

/* Functions in mouse.c */
void init_mouse(void);
void wait_for_mouse(void);
void show_mouse(void);
void delete_mouse(void);
void move_mouse(void);

/* Functions in callback.c */
void pfl_cb(int track_id);
void mute_cb(int track_id);
void edit_cb(int track_id);
void rec_cb(int track_id);
void switch_cb(int track_id);
void volume_cb(int button_pressed);
void volume_effect_cb(int slider_pressed);
void rew_cb(int data);
void stop_cb(int data);
void ff_cb(int data);
void mixer_cb(int data);
void load_cb(int data);
void save_cb(int data);
void send_cb(int data);
void scrollbar_cb(int data);
void zoomin_cb(int data);
void zoomout_cb(int data);
void zoom_cb(int data);
void record_cb(int data);
void play_cb(int data);
void monitor_cb(int data);
void mixer0_cb(int data);
void mixer1_cb(int data);
void cut_cb(int data);
void paste_cb(int data);
void copy_cb(int data);
void amplify_cb(int data);
void quit_cb(int data);
