/*************************************************************************
    record.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/soundcard.h>
#include "multitrack.h"

static int audio_fd;
static short int *input, *data_left, *data_right;


static void error_record(char *s)
{
	fprintf(stderr, "Error record: %s\n", s);
	exit(0);
}


static void record(void)
{
	int count, slicelen, channel, i;
	long position, total_length;
	int fd[2];
	char filename[80];

	channel = 0;
	for (count = 0; count < nr_tracks; count++)
		if (track[count].rec) {
			sprintf(filename, "track%d.wav", count);
			fd[channel] = open(filename, O_WRONLY);
			if (fd[channel] == -1)
				error_record("Cannot open track for writing.");
			lseek(fd[channel], (info->play_begin + 660) * 2, SEEK_SET);
			channel++;
		}
	if (soundcard[1].ok || soundcard[0].dual_dma)
		open_audio(&audio_fd, 1, O_RDONLY, channel - 1);
	else if (soundcard[0].ok)
		open_audio(&audio_fd, 0, O_RDONLY, channel - 1);

	if (soundcard[1].ok && soundcard[0].ok)
		while (info->time_count == info->play_begin)
			usleep(1);	/* wait for the playing section */

	position = 0;
	total_length = info->play_end - info->play_begin;
	do {
		slicelen = total_length - position;
		if (slicelen > soundcard[1].buffersize)
			slicelen = soundcard[1].buffersize;

		if (soundcard[1].ok || soundcard[0].ok)
			read(audio_fd, (char *) (input), slicelen * 2 * channel);
		if (channel == 1) {
			write(fd[0], (char *) input, slicelen * 2);
		} else {	/* stereo */
			for (i = 0; i < slicelen; i++) {
				data_left[i] = input[i * 2];
				data_right[i] = input[i * 2 + 1];
			}
			write(fd[0], (char *) data_left, slicelen * 2);
			write(fd[1], (char *) data_right, slicelen * 2);
		}
		position += slicelen;
		if (!info->playing)
			info->time_count = info->play_begin + position;
	}
	while (slicelen == soundcard[1].buffersize && info->recording);

	info->recording = FALSE;
	info->finished_recording = TRUE;

	close(fd[0]);
	if (channel == 2)
		close(fd[1]);
	if (soundcard[1].ok || soundcard[0].ok)
		close(audio_fd);
}


static void record_handler(int sig)
{
	if (sig == SIGUSR1) {
/*              fprintf(stderr, "Signal 1: Startig to record...\n"); */
		info->recording = TRUE;
		signal(SIGUSR1, record_handler);	/* reinstall handler */
	}
	if (sig == SIGUSR2) {
/*              fprintf(stderr, "Signal 2: Stop recording...\n"); */
		info->recording = FALSE;
		signal(SIGUSR2, record_handler);	/* reinstall handler */
	}
	return;
}


static void kill_handler(int sig)
{
	fprintf(stderr, "Terminating record child...\n");
	free(input);
	free(data_left);
	free(data_right);
}


void main_recordchild(void)
{
	input = (short int *) malloc(sizeof(short int) * soundcard[1].buffersize * 2);
	data_left = (short int *) malloc(sizeof(short int) * soundcard[1].buffersize);
	data_right = (short int *) malloc(sizeof(short int) * soundcard[1].buffersize);
	if (input == NULL || data_left == NULL || data_right == NULL)
		error_record("Unable to allocate data-array.");

	signal(SIGUSR1, record_handler);
	signal(SIGUSR2, record_handler);
	signal(SIGKILL, kill_handler);
	while (1) {
		pause();
		if (info->recording)
			record();
	}
}
