/*************************************************************************
    screen.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"


long screen_to_sample(int screen_x)
/* This function returns the position in the wave in sample units.
   screen_x is in screen units */
{
	return (long) (track_begin + (track_end - track_begin) *
	   (double) (screen_x - screen_min) / (screen_max - screen_min));
}


void gl_box(int x, int y, int width, int height, int color)
{
	vga_setcolor(color);
	vga_drawline(x, y, x + width, y);
	vga_drawline(x, y + height, x + width, y + height);
	vga_drawline(x, y, x, y + height);
	vga_drawline(x + width, y, x + width, y + height);
}


void open_screen(void)
{
	GraphicsContext physicalscreen;
	int VGAMODE = G640x480x256;

	vga_init();

	if (!vga_hasmode(VGAMODE))
		error("Mode not available.");

	vga_setmode(VGAMODE);
	gl_setcontextvga(VGAMODE);	/* Physical screen context. */
	gl_getcontext(&physicalscreen);
	gl_setrgbpalette();

	font = malloc(256 * 8 * 8 * BYTESPERPIXEL);
	gl_expandfont(8, 8, 255, gl_font8x8, font);
	gl_setfont(8, 8, font);
	gl_setwritemode(WRITEMODE_MASKED);

	gl_fillbox(0, 0, 639, 21, DARKGREY);	/* Button bar */
}


int track_window(void)
{
	if (mouse_x >= screen_min && mouse_x <= screen_max &&
	    mouse_y >= 43 && mouse_y <= 470)
		return TRUE;
	else
		return FALSE;
}


int scrollbar_window(void)
{
	if (mouse_x >= screen_min && mouse_x <= screen_max &&
	    mouse_y >= 30 && mouse_y <= 40)
		return TRUE;
	else
		return FALSE;
}


void select_region(int begin, int end, int select)
{
	int x, line, count, element;
	unsigned char fg_color, bg_color, sel_color = YELLOW;

	for (count = 0; count < nr_tracks; count++)
		if (count != bigbuf_info->show_buffer) {
			bg_color = BLACK;
			if (select) {
				if (track[count].rec)
					bg_color = RED;
				else
					bg_color = GREY;
				fg_color = sel_color;
			} else
				fg_color = GREEN;
			if (count == edit_track)
				if (select)
					fg_color = LIGHTBLUE;
				else
					fg_color = BLUE;

			for (line = 0; line <= screen[count].height; line++)
				for (x = begin - screen_min; x <= end - screen_min; x++) {
					element = line * screen_length + x;
					if (screen[count].data[element] != GREEN &&
					    screen[count].data[element] != LIGHTBLUE &&
					    screen[count].data[element] != BLUE &&
					    screen[count].data[element] != sel_color)
						screen[count].data[element] = bg_color;
					else
						screen[count].data[element] = fg_color;
				}
		}
	display_tracks(begin, end);
}


void select_track(void)
{
	int center;
	char hulp[80];

	delete_mouse();
	select_region(select_begin, select_end, FALSE);
	display_tracks(select_begin, select_end);
	show_mouse();
	select_begin = select_end = center = mouse_x;
	do {
		usleep(5000);
		if (mouse_update()) {
			delete_mouse();
			mouse_x = mouse_getx();
			mouse_y = mouse_gety();
			if (mouse_x > center) {
				if (select_begin != center) {
					select_region(select_begin, center, FALSE);
					select_begin = center;
				}
				if (mouse_x > select_end)
					if (mouse_x > screen_max)
						select_region(select_end, screen_max, TRUE);
					else
						select_region(select_end, mouse_x, TRUE);
				else
					select_region(mouse_x, select_end, FALSE);
				select_end = mouse_x;
				if (select_end > screen_max)
					select_end = screen_max;
			} else {
				if (select_end != center) {
					select_region(center, select_end, FALSE);
					select_end = center;
				}
				if (mouse_x < select_begin)
					if (mouse_x < screen_min)
						select_region(screen_min, select_begin, TRUE);
					else
						select_region(mouse_x, select_begin, TRUE);
				else
					select_region(select_begin, mouse_x, FALSE);
				select_begin = mouse_x;
				if (select_begin < screen_min)
					select_begin = screen_min;
			}
			sprintf(hulp, "%0.3f - %0.3f", screen_to_sample(select_begin) / 22050.0,
				screen_to_sample(select_end) / 22050.0);
			gl_fillbox(0, 27, 162, 9, BLACK);
			gl_fillbox(0, 27, strlen(hulp) * 8 + 2, 9, GREY);
			gl_write(1, 28, hulp);
			show_mouse();
		}
	}
	while (mouse_getbutton() == MOUSE_LEFTBUTTON);

	if (select_begin == select_end) {
		delete_mouse();
		select_begin = screen_min;
		select_end = screen_min;
		gl_fillbox(0, 27, 162, 9, BLACK);
		show_mouse();
	}
}


void zoom_in(void)
{
	if (track_end != track_begin + screen_length - 1) {
		track_end = track_begin + (track_end - track_begin + 1) / 2;
		if (track_end < track_begin + screen_length - 1)
			track_end = track_begin + screen_length - 1;
		select_begin = select_end = screen_min;
		gl_fillbox(0, 27, 162, 9, BLACK);
		show_tracks();
		show_scrollbar();
	}
}


void zoom_out(void)
{
	if (track_end != track_length - 1) {
		track_end = track_begin + (track_end - track_begin + 1) * 2;
		if (track_end > track_length - 1)
			track_end = track_length - 1;
		select_begin = select_end = screen_min;
		gl_fillbox(0, 27, 162, 9, BLACK);
		show_tracks();
		show_scrollbar();
	}
}


void zoom(void)
{
	int changed = FALSE;
	long begin, end;

	if ((track_end - track_begin + 1) != track_length &&	/* Full view */
	    select_begin == select_end) {
		track_begin = 0;
		track_end = track_length - 1;
		changed = TRUE;
	} else if (select_end != select_begin) {
		begin = screen_to_sample(select_begin);
		end = screen_to_sample(select_end);
		track_begin = begin;
		track_end = end;
		if (track_end < track_begin + screen_length - 1)
			track_end = track_begin + screen_length - 1;
		changed = TRUE;
	}
	if (changed) {
		delete_mouse();
		select_begin = select_end = screen_min;
		gl_fillbox(0, 27, 162, 9, BLACK);
		show_tracks();
		show_scrollbar();
		show_mouse();
	}
}


void show_scrollbar(void)
{
	int width;

	gl_box(screen_min - 1, 30, screen_length + 1, 4, BLUE);
	gl_fillbox(screen_min, 31, screen_length, 3, BLACK);
	width = (track_end - track_begin + 1) / (float) track_length *screen_length;
	if (width < 1)
		width = 1;
	gl_fillbox(screen_min + track_begin / (float) track_length * screen_length, 31,
		   width, 3, GREEN);
}


void move_scrollbar(void)
{
	long length;
	int bar_length, step;

	length = track_end - track_begin + 1;
	if (length != track_length) {
		bar_length = length / (float) track_length *screen_length;
		step = (int) (track_length / (float) screen_length);
		do {
			usleep(20000);
			if (mouse_update()) {
				delete_mouse();
				mouse_x = mouse_getx();
				mouse_y = mouse_gety();
				track_begin = (mouse_x - screen_min) * step;
				if (track_begin < 0)
					track_begin = 0;
				if (track_begin > track_length - length - 1)
					track_begin = track_length - length - 1;
				track_end = track_begin + length;
				show_scrollbar();
				show_mouse();
			}
		}
		while (mouse_getbutton() == MOUSE_LEFTBUTTON);

		select_begin = select_end = screen_min;
		gl_fillbox(0, 27, 162, 9, BLACK);
		delete_mouse();
		show_tracks();
		show_mouse();
	}
}
