/*************************************************************************
    slider.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"


static void move_slider(Slider * slid, int slider_pressed, void (*function) (void))
{
	int position;

	do {
		usleep(1000);
		if (info->playing && !sub_window)
			show_play_cursor(FALSE);
		function();
		if (mouse_update()) {
			delete_mouse();
			mouse_x = mouse_getx();
			mouse_y = mouse_gety();
			position = slid->length - (mouse_y - slid->y);
			if (position < 0)
				position = 0;
			if (position > slid->length)
				position = slid->length;
			slid->position = position;
			show_slider(*slid, FALSE);
			show_mouse();

			slid->function(slider_pressed);
		}
	}
	while (mouse_getbutton() == MOUSE_LEFTBUTTON);
}


int press_slider(Slider * slid, int number_sliders, void (*function) (void))
{
	int slider_pressed = -1, i;

	for (i = 0; i < number_sliders; i++)
		if (mouse_x >= slid[i].x && mouse_x <= slid[i].x + 15 &&
		    mouse_y >= (slid[i].y - 4) && mouse_y <= (slid[i].y + slid[i].length + 4))
			slider_pressed = i;

	if (slider_pressed != -1 && slid[slider_pressed].active)
		move_slider(&slid[slider_pressed], slider_pressed, function);

	return slider_pressed;
}


void show_slider(Slider slid, int pressed)
{
	gl_fillbox(slid.x, slid.y - 4, 15, slid.length + 8, DARKGREY);
	gl_box(slid.x + 5, slid.y - 1, 5, slid.length + 2, GREY);
	gl_box(slid.x + 7, slid.y, 1, slid.length, BLACK);
	if (slid.middle)
		gl_box(slid.x + 5, (slid.length) / 2 + slid.y, 6, 0, RED);
	gl_fillbox(slid.x, slid.length - slid.position + slid.y - 3, 15, 7, GREY);
	gl_box(slid.x + 1, slid.length - slid.position + slid.y, 12, 0, YELLOW);
}


void show_track_sliders(int track_id)
{
	int slider_id, volume;

	slider_id = LR_SLIDE + track_id;
	if (track[track_id].volume_l > track[track_id].volume_r)
		volume = track[track_id].volume_l;
	else
		volume = track[track_id].volume_r;
	if (slider[slider_id].active) {
		slider[slider_id].position = (int) (((track[track_id].volume_l - track[track_id].volume_r) / (float) volume + 1) *
					 (slider[slider_id].length) / 2);
		show_slider(slider[slider_id], FALSE);
		gl_write(slider[slider_id].x + 15, slider[slider_id].y, "L");
		gl_write(slider[slider_id].x + 15, slider[slider_id].y + slider[slider_id].length - 4, "R");
	}
	slider_id = VOL_SLIDE + track_id;
	if (slider[slider_id].active) {
		slider[slider_id].position = (int) (volume / 64.0 *
					       slider[slider_id].length);
		show_slider(slider[slider_id], FALSE);
		gl_write(slider[slider_id].x - 3, slider[slider_id].y - 13, "Vol");
	}
	slider_id = EFFECT_SLIDE + track_id;
	if (slider[slider_id].active) {
		slider[slider_id].position = track[track_id].volume_effect;
		show_slider(slider[slider_id], FALSE);
		gl_write(slider[slider_id].x - 8, slider[slider_id].y - 13, "Send");
	}
}


void init_sliders(void)
{
	int count, slider_id;

	for (count = 0; count < nr_tracks; count++) {
		slider_id = LR_SLIDE + count;
		slider[slider_id].x = 135;
		slider[slider_id].y = screen[count].position + 30;
		slider[slider_id].active = TRUE;
		slider[slider_id].length = 64;
		slider[slider_id].middle = TRUE;
		slider[slider_id].function = volume_cb;

		slider_id = VOL_SLIDE + count;
		slider[slider_id].x = 165;
		slider[slider_id].y = screen[count].position + 30;
		slider[slider_id].active = TRUE;
		slider[slider_id].length = 64;
		slider[slider_id].middle = FALSE;
		slider[slider_id].function = volume_cb;

		slider_id = EFFECT_SLIDE + count;
		slider[slider_id].x = 80;
		slider[slider_id].y = screen[count].position + 30;
		slider[slider_id].active = TRUE;
		slider[slider_id].length = 64;
		slider[slider_id].middle = FALSE;
		slider[slider_id].function = volume_effect_cb;
	}
}
