/*************************************************************************
    track.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <vga.h>
#include <vgagl.h>
#include "multitrack.h"


void init_tracks(void)
{
	int count, step, fd;
	struct stat buf;

	/* determine track-length */
	fd = open("track0.wav", O_RDONLY);
	if (fd == -1)
		error("Cannot open track0.wav.");
	fstat(fd, &buf);	/* find length of track 0 */
	track_length = buf.st_size / 2;		/* 16 bit samples */
	close(fd);

	track_begin = 0;
	track_end = track_length - 1;
	edit_track = -1;

	info->time_count = 0;
	info->play_begin = track_begin;
	info->play_end = track_end;
	info->effect_send = FALSE;
	info->playing = FALSE;
	info->finished_playing = FALSE;
	info->recording = FALSE;
	info->finished_recording = FALSE;
	info->ff = FALSE;
	info->rew = FALSE;

	bigbuf_info->filled = FALSE;
	bigbuf_info->length = 0;
	bigbuf_info->position = track_begin;
	bigbuf_info->volume = 40;
	bigbuf_info->show_buffer = -1;
	bigbuf_info->reverse = FALSE;

	track[0].volume_l = 64;	/* set left volume */
	track[0].volume_r = 0;	/* set right volume */
	track[0].volume_effect = 0;
	track[1].volume_l = 0;
	track[1].volume_r = 64;
	track[1].volume_effect = 0;
	track[2].volume_l = 64;
	track[2].volume_r = 64;
	track[2].volume_effect = 0;
	track[3].volume_l = 40;
	track[3].volume_r = 40;
	track[3].volume_effect = 0;

	step = 400 / nr_tracks;
	for (count = 0; count < nr_tracks; count++) {
		track[count].mute = FALSE;
		track[count].pfl = FALSE;
		track[count].rec = FALSE;

		/* Allocate screen memory for every track.... */
		screen[count].height = step;
		screen[count].position = 44 + (step + 8) * count;
		screen[count].data = (unsigned char *) malloc(sizeof(char) * (step + 1) * screen_length);
		if (screen[count].data == NULL)
			error("Cannot allocate data array.");
	}
}


void show_track_window(int count)
{
	char hulp[20];

	delete_mouse();
	/* Put a box of 441 pixels length (2 seconds sample time) */
	gl_box(screen_min - 1, screen[count].position - 1,
	       screen_length + 1, screen[count].height + 2, BLUE);

	/* window for buttons per track */
	gl_fillbox(2, screen[count].position,
		   screen_min - 8, screen[count].height, DARKGREY);
	gl_box(0, screen[count].position - 2, screen_min - 4, screen[count].height + 4, GREY);
	gl_box(1, screen[count].position - 1, screen_min - 6, screen[count].height + 2, GREY);
	gl_fillbox(54, screen[count].position + 2, screen_min - 62, 11, DARKGREEN);
	if (bigbuf_info->show_buffer != count) {
		sprintf(hulp, "Track %d", count);
		button[PFL + count].active = TRUE;
		button[MUTE + count].active = TRUE;
		button[EDIT + count].active = TRUE;
		button[REC + count].active = TRUE;
		button[SWITCH + count].active = TRUE;
		slider[LR_SLIDE + count].active = TRUE;
		slider[EFFECT_SLIDE + count].active = button[SEND].pressed;
	} else {
		sprintf(hulp, "Buffer");
		button[PFL + count].active = FALSE;
		button[MUTE + count].active = FALSE;
		button[EDIT + count].active = FALSE;
		button[REC + count].active = FALSE;
		button[SWITCH + count].active = TRUE;
		slider[LR_SLIDE + count].active = FALSE;
		slider[EFFECT_SLIDE + count].active = FALSE;
	}
	gl_write(54 + (130 - strlen(hulp) * 8) / 2, screen[count].position + 4, hulp);

	show_track_buttons(count);
	show_track_sliders(count);
	show_mouse();
}


int tracks_selected(void)
{
	int nr = 0, count;

	for (count = 0; count < nr_tracks; count++)
		if (track[count].rec)
			nr++;
	return nr;
}


void display_tracks(int begin, int end)
{
	int count, line;

	if (begin < screen_min)
		begin = screen_min;
	if (end > screen_max)
		end = screen_max;

	for (count = 0; count < nr_tracks; count++) {
		for (line = 0; line <= screen[count].height; line++)
			vga_drawscansegment(screen[count].data + line * screen_length + begin - screen_min, begin,
					    screen[count].position + line,
					    end - begin + 1);
	}
}


void max_min(short int *buffer, int number, char *output_buf)
{
	short int min, max;
	int i;

	/* determine maximum and minimum inside block */
	if (number > 0) {
		max = min = buffer[0];
		for (i = 0; i < number; i++)
			if (buffer[i] > max)
				max = buffer[i];
			else if (buffer[i] < min)
				min = buffer[i];

		output_buf[0] = (char) (min >> 8);	/* fill output buffer */
		output_buf[1] = (char) (max >> 8);
	}
}


void make_display_file(int track_nr, long begin, long end)
{
	int input_fd, output_fd, slicelen;
	char filename[80], hulp[80];
	char write_buf[2];
	short int *buf;
	long position;

	buf = (short int *) malloc(sizeof(short int) * blocksize);
	if (buf == NULL)
		error("Cannot allocate read buffer.");

	sprintf(filename, "track%d.wav", track_nr);
	input_fd = open(filename, O_RDONLY);
	if (input_fd == -1)
		error("Cannot open track for reading.");

	/* Adjust begin and end to a multiple of blocksize */
	begin = (begin / blocksize) * blocksize - blocksize;
	if (begin < 0)
		begin = 0;
	end = (end / blocksize) * blocksize + blocksize;
	if (end > track_length)
		end = track_length;

	lseek(input_fd, begin * 2, SEEK_SET);

	sprintf(filename, "display%d.dat", track_nr);
	output_fd = open(filename, O_CREAT | O_WRONLY, 0666);
	if (output_fd == -1)
		error("Cannot open display file for writing.");
	lseek(output_fd, (begin / blocksize) * 2, SEEK_SET);

	sprintf(hulp, "Reading track %d.", track_nr);
	init_progress(hulp);
	position = 0;
	do {
		slicelen = (end - begin) - position;
		if (slicelen >= blocksize) {
			slicelen = blocksize;
			read(input_fd, (char *) buf, slicelen * 2);	/* read bytes */
			max_min(buf, slicelen, write_buf);
			write(output_fd, write_buf, 2);		/* write maximum and */
			/* minimum. */
			position += slicelen;
		}
		show_progress(position / (float) (end - begin));
	}
	while (slicelen == blocksize);
	delete_progress();

	close(input_fd);
	close(output_fd);

	free(buf);
}


void show_tracks(void)
{
	int count, fd[nr_tracks], x, i, upper, lower, element, step;
	int max_step = 128, color;
	long position;
	char buffer[2], tmpbuffer[2], filename[80], hulp1[10], hulp2[10];
	short int read_buf[256];

	step = screen_to_sample(screen_min + 1) - screen_to_sample(screen_min);
	if (step > max_step) {
		for (count = 0; count < nr_tracks; count++) {
			sprintf(filename, "display%d.dat", count);
			fd[count] = open(filename, O_RDONLY);
			if (fd[count] == -1) {
				make_display_file(count, 0, track_length);
				fd[count] = open(filename, O_RDONLY);
			}
		}
	} else {
		for (count = 0; count < nr_tracks; count++) {
			sprintf(filename, "track%d.wav", count);
			fd[count] = open(filename, O_RDONLY);
			if (fd[count] == -1)
				error("Cannot open track file.");
		}
	}

	for (count = 0; count < nr_tracks; count++) {
		if (count != bigbuf_info->show_buffer) {
			for (x = screen_min; x <= screen_max; x++) {
				position = screen_to_sample(x);
				if (step > max_step) {
					lseek(fd[count], (position / 512) * 2, SEEK_SET);
					read(fd[count], buffer, 2);
					for (i = 0; i < step / 512; i++) {
						read(fd[count], tmpbuffer, 2);
						if (tmpbuffer[0] < buffer[0])
							buffer[0] = tmpbuffer[0];
						if (tmpbuffer[1] > buffer[1])
							buffer[1] = tmpbuffer[1];
					}
				} else {
					lseek(fd[count], position * 2, SEEK_SET);
					read(fd[count], (char *) read_buf, step * 2);
					max_min(read_buf, step, buffer);
				}
				if (count == edit_track)
					color = BLUE;
				else
					color = GREEN;
				/* draw line */
				lower = (int) ((buffer[0] / 128.0 + 1) * screen[count].height / 2);
				upper = (int) ((buffer[1] / 128.0 + 1) * screen[count].height / 2);
				for (i = 0; i <= screen[count].height; i++) {
					element = i * screen_length + x - screen_min;
					if (i >= lower && i <= upper)
						screen[count].data[element] = color;
					else
						screen[count].data[element] = BLACK;
				}
			}
		} else
			show_buffer(count);
	}
	for (count = 0; count < nr_tracks; count++)
		close(fd[count]);

	display_tracks(screen_min, screen_max);
	gl_fillbox(screen_min - 2, 471, screen_length + 2, 7, BLACK);
	if ((track_end - track_begin) / 22050.0 < 1) {
		sprintf(hulp1, "%0.3f", track_begin / 22050.0);
		sprintf(hulp2, "%7.3f", track_end / 22050.0);
	} else if ((track_end - track_begin) / 22050.0 < 10) {
		sprintf(hulp1, "%0.2f", track_begin / 22050.0);
		sprintf(hulp2, "%6.2f", track_end / 22050.0);
	} else {
		sprintf(hulp1, "%0.1f", track_begin / 22050.0);
		sprintf(hulp2, "%5.1f", track_end / 22050.0);
	}
	gl_write(screen_min - 2, 471, hulp1);
	gl_write(screen_max - strlen(hulp2) * 8, 471, hulp2);
}
