/*************************************************************************
    window.c -- last change: 11-11-1996

    Multitrack is a program to play, record and manipulate tracks. The
    tracks are stored on disk.

    Copyright (C) 1996  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <vgagl.h>
#include <vgamouse.h>
#include "multitrack.h"

static char *screen_buffer;

static void show_window(Window * win)
{
	/* allocate buffer to save screen where menu will appear */
	screen_buffer = (char *) malloc(sizeof(char) * (win->width + 9) * (win->height + 9));
	if (screen_buffer == NULL)
		error("Cannot allocate screen buffer.");

	gl_getbox(win->x - 5, win->y - 5, win->width + 9, win->height + 9, screen_buffer);
	gl_fillbox(win->x - 4, win->y - 4, win->width + 7, win->height + 7, DARKGREY);
	gl_box(win->x - 5, win->y - 5, win->width + 8, win->height + 8, BLACK);
	gl_box(win->x - 4, win->y - 4, win->width + 7, win->height + 7, BLACK);
	gl_box(win->x - 3, win->y - 3, win->width + 5, win->height + 5, GREY);
	gl_box(win->x - 2, win->y - 2, win->width + 3, win->height + 3, GREY);
	gl_box(win->x - 1, win->y - 1, win->width + 1, win->height + 1, BLACK);
}


static void delete_window(Window * win)
{
	/* restore previous screen */
	gl_putbox(win->x - 5, win->y - 5, win->width + 9, win->height + 9, screen_buffer);
	free(screen_buffer);
}


int do_window(Window * win, Button * but, int nr_but, Slider * slid,
	      int nr_slid, Window_text * text, int nr_text,
	      int stop_button, int cancel_button)
{
	int stop = FALSE, ok = FALSE, i, button_id;

	sub_window = TRUE;
	delete_mouse();
	show_window(win);
	show_mouse();
	for (i = 0; i < nr_but; i++)
		show_button(&but[i], but[i].pressed);
	for (i = 0; i < nr_slid; i++)
		show_slider(slid[i], FALSE);
	delete_mouse();
	for (i = 0; i < nr_text; i++)
		gl_write(text[i].x, text[i].y, text[i].string);
	show_mouse();
	do {
		usleep(10000);
		win->function();
		if (mouse_update()) {
			delete_mouse();
			mouse_x = mouse_getx();
			mouse_y = mouse_gety();
			show_mouse();
			if (mouse_getbutton() == MOUSE_LEFTBUTTON) {
				button_id = press_button(but, nr_but);
				if (button_id == stop_button) {
					stop = TRUE;
					ok = TRUE;
				}
				if (button_id == cancel_button) {
					stop = TRUE;
					ok = FALSE;
				}
				if (nr_slid != 0)
					press_slider(slid, nr_slid, win->function);
			}
		}
	}
	while (!stop);

	delete_mouse();
	delete_window(win);
	show_mouse();
	sub_window = FALSE;

	return ok;
}


static Window win;

void init_progress(char *string)
{
	win.x = 270;
	win.y = 225;
	win.width = 210;
	win.height = 60;

	delete_mouse();
	show_window(&win);
	gl_write(win.x + 5, win.y + 5, string);
	gl_write(win.x + 5, win.y + 30, "Progress:");
	gl_fillbox(win.x + 4, win.y + 39, 201, 11, WHITE);
	gl_box(win.x + 4, win.y + 39, 201, 11, BLACK);
}


void show_progress(float percent)
{
	gl_fillbox(win.x + 5, win.y + 40, 2 * (int) (percent * 100), 10, LIGHTBLUE);
}


void delete_progress(void)
{
	delete_window(&win);
	show_mouse();
}
