/*************************************************************************
    dialog.c -- last change: 20-1-1998

    Copyright (C) 1996-1998  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "local.h"
#include "images/dia_exclamation.xpm"
#include "images/dia_info.xpm"
#include "images/dia_question.xpm"
#include "images/dia_stop.xpm"


static void add_symbol(GuiWindow *win, int symbol)
{
	if (symbol == 0)
		return;
		
	check_window(win, "add_symbol");

	switch(symbol) {
		case DIA_EXCLAMATION :
			add_image(win, 10, (win->height - 32) / 2 + 5, dia_exclamation);
			break;
		case DIA_INFO :
			add_image(win, 10, (win->height - 32) / 2 + 5, dia_info);
			break;
		case DIA_QUESTION :
			add_image(win, 10, (win->height - 32) / 2 + 5, dia_question);
			break;
		case DIA_STOP :
			add_image(win, 10, (win->height - 32) / 2 + 5, dia_stop);
			break;
	}
}


int question_dialog(GuiWindow *p_win, char *message, int symbol)
{
	GuiWinThread *win_thread;
	GuiWindow *win;
	GuiObject *obj = NULL, *Yes, *No;
	int width, height, x, y, pos = 0, count = 0;
	char *hulp, *line;

	determine_geometry(message, &width, &height);
	width += 16;
	height += 50;

	if (width < 98)
		width = 98;
	if (symbol != 0)
		pos = 52;
	width += pos;

	if (p_win == NULL) {	/* in the center of the screen */
		x = (guiscreen.width - width) / 2;
		y = (guiscreen.height - height) / 2;
	} else {		/* in the center of the parent window */
		x = p_win->x + (p_win->width - width) / 2;
		y = p_win->y + (p_win->height - height) / 2;
	}
	win_thread = create_window_thread();
	win = add_window(win_thread, NORMAL_WINDOW, x, y, width, height, "Question", FALSE, FALSE);

	hulp = (char *)malloc(strlen(message) + 1);
	if (hulp == NULL)
		error("Cannot allocate memory in question_dialog().");
        sprintf(hulp, "%s", message);
	line = strtok(hulp, "\n");
	while (line != NULL) {
		add_text(win, NORMAL_TEXT, 5 + pos, 20 + count * 13, line);
		line = strtok(NULL, "\n");
		count++;
	}
	free(hulp);

	add_symbol(win, symbol);

	No = add_button(win, NORMAL_BUTTON, win->width - 49, win->height - 25, 44, 20, "No");
	Yes = add_button(win, NORMAL_BUTTON, 5 + pos, win->height - 25, 44, 20, "Yes");

	create_window(win);
	set_mouse_position(x + (width + pos) / 2, y + Yes->y + 10);
	move_mouse();

	show_window_thread(win_thread);
	while (obj != No && obj != Yes)
		obj = do_windows(win_thread);
	delete_window_thread(win_thread);

	if (obj == Yes)
		return TRUE;
	else
		return FALSE;
}


void message_dialog(GuiWindow *p_win, char *message, int symbol)
{
	GuiWinThread *win_thread;
	GuiWindow *win;
	GuiObject *obj = NULL, *ok;
	int x, y, width, height, pos = 0, count = 0;
	char *hulp, *line;

	determine_geometry(message, &width, &height);
	width += 16;
	height += 50;
	if (width < 58)
		width = 58;
	if (symbol != 0)
		pos = 52;
	width += pos;
		
	if (p_win == NULL) {	/* in the center of the screen */
		x = (guiscreen.width - width) / 2;
		y = (guiscreen.height - height) / 2;
	} else {		/* in the center of the parent window */
		x = p_win->x + (p_win->width - width) / 2;
		y = p_win->y + (p_win->height - height) / 2;
	}
	win_thread = create_window_thread();
	win = add_window(win_thread, NORMAL_WINDOW, x, y, width, height, "Message", FALSE, FALSE);

	hulp = (char *)malloc(strlen(message) + 1);
	if (hulp == NULL)
		error("Cannot allocate memory in message_dialog().");
        sprintf(hulp, "%s", message);
	line = strtok(hulp, "\n");
	count = 0;
	while (line != NULL) {
		add_text(win, NORMAL_TEXT, 5 + pos, 20 + count * 13, line);
		line = strtok(NULL, "\n");
		count++;
	}
	free(hulp);
	add_symbol(win, symbol);

	ok = add_button(win, NORMAL_BUTTON, (win->width - pos) / 2 - 22 + pos,
			win->height - 25, 44, 20, "Ok");
	win->enterobj = ok;

	create_window(win);
	set_mouse_position(x + ok->x + ok->width / 2, y + ok->y + ok->height / 2);
	move_mouse();
	
	show_window_thread(win_thread);	
	while (obj != ok)
		obj = do_windows(win_thread);
	delete_window_thread(win_thread);
}
