/*************************************************************************
    filedialog.c -- last change: 20-1-1998

    Copyright (C) 1996-1998  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include "local.h"

static GuiObject *ok, *cancel, *filter, *filter_name, *file_name;
static GuiObject *dir_list, *dir_text, *file_list, *file_text;
static GuiObject **dir_obj, **file_obj;
static int nr_dirs, nr_files, double_click;
static int create_lists(GuiWindow *win);


static void set_file_cb(GuiObject *obj, int data)
{
	check_object(obj, "obj", "set_file_cb");

	set_input(file_name, obj->label);
	show_input(file_name);
}


static void reread_dir(GuiWindow *win)
{
	char hulp[256], hulp2[256], abs_path[256];
	int i;
	
	check_window(win, "reread_dir");
	
	sprintf(hulp, "%s", filter_name->label);
	if (chdir(hulp) == -1) {
		sprintf(hulp2, "Cannot change to directory\n%s.", hulp);
		message_dialog(win, hulp2, DIA_STOP);
		getcwd(abs_path, 256);
		sprintf(hulp, "%s/", abs_path);
		set_input(filter_name, hulp);
		show_input(filter_name);
		return;
	}
	/* delete directory objects */
	win_fillbox(win, dir_list->x - 2, dir_list->y - 2, dir_list->width + 4,
		    dir_list->height + 4, WIN_BACK);
	for (i = 0;i < nr_dirs;i++)
		delete_object(dir_obj[i]);
	free(dir_obj);
	dir_list->tot_nr_items = 0;
	
	/* delete file objects */
	win_fillbox(win, file_list->x - 2, file_list->y - 2, file_list->width + 4,
		    file_list->height + 4, WIN_BACK);
	win_fillbox(win, file_text->x, file_text->y, file_text->width,
		    file_text->height, WIN_BACK);
	for (i = 0;i < nr_files;i++)
		delete_object(file_obj[i]);
	free(file_obj);
	file_list->tot_nr_items = 0;

	/* read the new directory */
	create_lists(win);
	/* update the directory path */
	getcwd(abs_path, 256);
	sprintf(hulp, "%s/", abs_path);
	set_input(filter_name, hulp);
	show_input(filter_name);
	update_objects(win);
}


static void filter_cb(GuiObject *obj, int data)
{
	int i;
	char hulp[256];
	
	check_object(obj, "obj", "filter_cb");
	check_window(obj->win, "filter_cb");
	
	for (i = 0;i < nr_dirs;i++)
		if (dir_obj[i]->pressed) {
			sprintf(hulp, "%s%s", filter_name->label, dir_obj[i]->label);
			set_input(filter_name, hulp);
			show_input(filter_name);
		}
	reread_dir(obj->win);
	show_button(obj);
}


static void reread_dir_cb(GuiObject *obj, int data)
{
	GuiWindow *win;
	char hulp[256];

	check_object(obj, "obj", "reread_dir_cb");
	win = obj->win;
	check_window(win, "reread_dir_cb");

	if (obj->objclass == LISTENTRY) {
		sprintf(hulp, "%s%s", filter_name->label, obj->label);
		set_input(filter_name, hulp);
		show_input(filter_name);
	}
	reread_dir(win);
	show_window(win);
}


static void make_index(char **name, int *index, int nr)
{
	int i, count, count2;
	
	index[0] = 0;
	for (i = 1;i < nr;i++) {
		count = 0;
		while (strcmp(name[i], name[index[count]]) > 0) {
			count++;
			if (count >= i)
				break;
		}
		for (count2 = i - 1;count2 >= count;count2--)
			index[count2 + 1] = index[count2];
		index[count] = i;
	}
}


static void exit_dialog_cb(GuiObject *obj, int data)
{
	check_object(obj, "obj", "exit_dialog_cb");

	double_click = TRUE;
}


static int create_lists(GuiWindow *win)
{
	DIR *dirp;
	struct dirent *dp;
	struct stat sbuf;
	char **directories, **files, *name;
	int i, dir_index[100], file_index[100];
	
	check_window(win, "create_lists");

	nr_dirs = 0;
	nr_files = 0;
	directories = (char **)malloc(sizeof(char *) * 100);
	files = (char **)malloc(sizeof(char *) * 100);
	if (directories == NULL || files == NULL)
		error("Cannot allocate memory in create_lists().");
	for (i = 0;i < 100;i++) {
		directories[i] = (char *)malloc(30);
		files[i] = (char *)malloc(30);
		if (directories[i] == NULL || files[i] == NULL)
			error("Cannot allocate memory in create_lists().");
	}

	dirp = opendir(".");
	if (dirp == NULL) {
		fprintf(stderr, "---> Warning: cannot open directory.\n");
		return FALSE;
	}
	
	dp = readdir(dirp);
	while (dp != NULL && nr_dirs < 98 && nr_files < 100) {
		name = dp->d_name;
		if (stat(name, &sbuf) == 0) {
			if (name[0] != '.') {
				if (S_ISDIR(sbuf.st_mode)) {
					sprintf(directories[nr_dirs], "%s", name);
					nr_dirs++;
				} else {
					sprintf(files[nr_files], "%s", name);
					nr_files++;
				}
			}
		} else
			fprintf(stderr, "Cannot read file attributes of %s\n", name);
		dp = readdir(dirp);
	}
	closedir(dirp);
	
	make_index(directories, dir_index, nr_dirs);
	make_index(files, file_index, nr_files);

	dir_obj = (GuiObject **)malloc(sizeof(GuiObject *) * (nr_dirs + 2));
	if (dir_obj == NULL)
		error("Cannot allocate dir_obj[] in create_lists().");
	dir_obj[0] = add_listentry(dir_list, ".");
	set_object_callback2(dir_obj[0], reread_dir_cb);
	dir_obj[1] = add_listentry(dir_list, "..");
	set_object_callback2(dir_obj[1], reread_dir_cb);
	for (i = 0;i < nr_dirs;i++) {
		dir_obj[i + 2] = add_listentry(dir_list, directories[dir_index[i]]);
		set_object_callback2(dir_obj[i + 2], reread_dir_cb);
	}
	nr_dirs += 2;
	create_listbox(dir_list);

	file_list->x = dir_list->x + dir_list->width + 30;
	file_text->x = dir_list->x + dir_list->width + 30;
	file_obj = (GuiObject **)malloc(sizeof(GuiObject *) * nr_files);
	if (file_obj == NULL)
		error("Cannot allocate file_obj[] in create_lists().");
	for (i = 0;i < nr_files;i++) {
		file_obj[i] = add_listentry(file_list, files[file_index[i]]);
		set_object_callback(file_obj[i], set_file_cb);
		set_object_callback2(file_obj[i], exit_dialog_cb);
	}
	create_listbox(file_list);

	free(directories);
	free(files);
	return TRUE;
}


static void init_file_interface(GuiWindow *win)
{
	int y = 60;
	
	check_window(win, "init_file_interface");

	add_text(win, NORMAL_TEXT, 10, 20, "Filter:");
	filter_name = add_input(win, NORMAL_INPUT, 10, 35, win->width - 30, 256);
	set_object_callback(filter_name, reread_dir_cb);
	add_text(win, NORMAL_TEXT, 10, win->height - 59, "File:");
	file_name = add_input(win, NORMAL_INPUT, 40, win->height - 60, 240, 256);

	dir_text = add_text(win, NORMAL_TEXT, 10, y, "Directories:");
	dir_list = add_listbox(win, 10, y + 15, 80, 8);
	file_text = add_text(win, NORMAL_TEXT, 110, y, "Files:"); 
	file_list = add_listbox(win, 110, y + 15, 100, 8); 
	create_lists(win);
	ok = add_button(win, NORMAL_BUTTON, 5, win->height - 25, 50, 20, "OK");
	filter = add_button(win, NORMAL_BUTTON, (win->width - 50) / 2, win->height - 25, 50, 20, "Filter");
	set_object_callback(filter, filter_cb);
	cancel = add_button(win, NORMAL_BUTTON, win->width - 55, win->height - 25, 50, 20, "Cancel");
}


int file_dialog(char *title, char *directory, char *filename, char *mask)
{
	GuiWinThread *win_thread;
	GuiWindow *win;
	GuiObject *obj = NULL;
	int width = 400, height = 270;
	int x = (guiscreen.width - width) / 2;
	int y = (guiscreen.height - height) / 2;
	int ret_value = FALSE;
	char hulp[256], hulp2[256], abs_path[256];

	if (strlen(directory) == 0) {
		getcwd(abs_path, 128);
		sprintf(hulp, "%s/", abs_path);
	} else
		sprintf(hulp, "%s", directory);
	if (chdir(hulp) == -1) {
		sprintf(hulp2, "Cannot change to directory\n%s.", hulp);
		message_dialog(NULL, hulp2, DIA_STOP);
		return FALSE;
	}

	win_thread = create_window_thread();
	win = add_window(win_thread, NORMAL_WINDOW, x, y, width, height,
			 title, TRUE, FALSE);

	init_file_interface(win);
	create_window(win);
	show_window_thread(win_thread);
	
	set_input(filter_name, hulp);
	show_input(filter_name);
	
	double_click = FALSE;
	while (obj != ok && obj != cancel && obj != win->kill && !double_click)
		obj = do_windows(win_thread);

	if (obj == ok || double_click) {
		ret_value = TRUE;
		sprintf(directory, "%s", filter_name->label);
		sprintf(filename, "%s", file_name->label);
	}
	delete_window_thread(win_thread);

	return ret_value;
}
