/*************************************************************************
    progress.c -- last change: 20-1-1998

    Copyright (C) 1996-1998  Boris Nagels

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    Sent your comments and suggestions to Boris Nagels, E-mail:
    boris@molphys.leidenuniv.nl

 *************************************************************************/

#include <stdio.h>
#include "local.h"

static GuiWinThread *prog_thread;
static GuiWindow *prog_win;
static GuiObject *perc_obj;
static int old_cursor;
static int progress_ok = FALSE;

void init_progress(char *string)
{
	int width = 212, height = 80;
	int x, y = (guiscreen.height - height) / 2;

	if (progress_ok) {
		fprintf(stderr, "---> Warning: progress() already exists.\n");
		return;
	}
	progress_ok = TRUE;
	old_cursor = mouse.cursor_id;
	switch_cursor(WATCH_CURSOR);
	if (string_length(string) + 10 > width)
		width = string_length(string) + 10;
	x = (guiscreen.width - width) / 2;

	prog_thread = create_window_thread();
	prog_win = add_window(prog_thread, NORMAL_WINDOW, x, y, width, height,
			 "Progress indication", FALSE, FALSE);

	add_text(prog_win, NORMAL_TEXT, 4, 25, string);
	add_text(prog_win, NORMAL_TEXT, 4, 45, "Progress:");
	perc_obj = add_text(prog_win, NORMAL_TEXT, 60, 45, "100%");
	create_window(prog_win);
	win_fillbox(prog_win, 6, 64, 200, 10, WHITE);
	win_3dbox(prog_win, DOWN_FRAME, 4, 62, 204, 14);
	show_window_thread(prog_thread);
}


void show_progress(float percent)
{
	char hulp[20];
	
	if (!progress_ok) {
		fprintf(stderr, "---> Warning: show_progress() called, but progress does not exist.\n");
		return;
	}
	win_fillbox(prog_win, 6, 64, 2 * (int) (percent * 100), 10, DARKBLUE);
	sprintf(hulp, "%0.0f%%", percent * 100);
	set_text(perc_obj, hulp);
	show_text(perc_obj);
	if (GuiGetMessage() == GuiMouseEvent)
		move_mouse();
}


void delete_progress(void)
{
	if (!progress_ok) {
		fprintf(stderr, "---> Warning: delete_progress() called, but progress does not exist.\n");
		return;
	}
	progress_ok = FALSE;
	delete_window_thread(prog_thread);
	switch_cursor(old_cursor);
}
